@@ -1,28 +1,18 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.c 2.15 1998/11/12 11:24:47 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.c 2.19 2000/05/14 13:51:53 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.4.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Generate error messages in a standard format optionally to syslog and stderr.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
   ***************************************/
 
 
-/*+ Under Linux, FreeBSD, NetBSD, Solaris x86, SGI, Ultrix, AIX, AT&T and Cygwin stdarg is used +*/
-#if defined(__linux__) || defined(hpux) || \
-    defined(__FreeBSD__) || defined(__NetBSD__) || \
-    (defined(__i386__) && defined(__sun__) && defined(__svr4__)) || \
-    defined(__sgi__) || defined(__sgi) || defined(__ultrix__) || \
-    defined(_AIX) || defined(_ATT4) || \
-    defined(__CYGWIN__)
-#define USE_STD_ARG
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,7 +22,7 @@
 #include <sys/types.h>
 #include <sys/fcntl.h>
 
-#ifdef USE_STD_ARG
+#ifdef __STDC__
 #include <stdarg.h>
 #else
 #include <varargs.h>
@@ -98,7 +88,7 @@ char* str_h_error(int err)
 #endif
 #ifdef HOST_NOT_FOUND
  if(err==HOST_NOT_FOUND)
-    return("Name Lookup Authoritative Answer Host not found");
+    return("Name Lookup Authoritive Answer Host not found");
  else
 #endif
 #ifdef TRY_AGAIN
@@ -151,6 +141,10 @@ static int ErrorPriority[]={-1          ,LOG_DEBUG,LOG_INFO     ,LOG_NOTICE ,LOG
 static int use_syslog=0,        /*+ use syslog. +*/
            use_stderr=1;        /*+ use stderr. +*/
 
+/*+ The level of error logging +*/
+ErrorLevel LogLevel=Important,  /*+ in the config file for syslog and stderr. +*/
+           DebugLevel=-1;       /*+ on the command line for stderr. +*/
+
 
 /*++++++++++++++++++++++++++++++++++++++
   Initialise the error handler, get the program name and pid.
@@ -232,7 +226,7 @@ char *PrintMessage(ErrorLevel errlev,const char* fmt, ...)
 
  string=(char*)malloc(str_len);
 
-#ifdef USE_STD_ARG
+#ifdef __STDC__
  va_start(ap,fmt);
 #else
  va_start(ap);
@@ -261,7 +255,7 @@ char *PrintMessage(ErrorLevel errlev,const char* fmt, ...)
           break;
 
          case 'c':
-          str[0]=va_arg(ap,char);
+          str[0]=(char)va_arg(ap,int); /* beware of type promotion */
           str[1]=0;
           strp=str;
           break;
@@ -289,15 +283,22 @@ char *PrintMessage(ErrorLevel errlev,const char* fmt, ...)
        j+=strlen(strp);
       }
 
+#if defined(__CYGWIN__)
+ if(string[j-1]=='\n')
+    j--;
+ string[j++]='\r';
+ string[j++]='\n';
+#else
  if(string[j-1]!='\n')
     string[j++]='\n';
+#endif
  string[j]=0;
 
  va_end(ap);
 
  /* Output the result. */
 
- if(UseSyslog && use_syslog && errlev>=LogLevel && ErrorPriority[errlev]!=-1)
+ if(use_syslog && errlev>=LogLevel && ErrorPriority[errlev]!=-1)
     syslog(ErrorPriority[errlev],"%s",string);
 
  if(use_stderr && ((DebugLevel==-1 && errlev>=LogLevel) || (DebugLevel!=-1 && errlev>=DebugLevel)))
@@ -351,7 +352,7 @@ static void closelog(void)
 static void syslog(int level,const char* format,char* string)
 {
  if(syslog_file!=-1 && syslog_facility)
-    write_formatted(syslog_file,"%s(%s): %s",syslog_facility,ErrorString[level],string);
+    write_formatted(syslog_file,"%s[%ld](%s): %s",syslog_facility,(long)pid,ErrorString[level],string);
 }
 
 #endif /* __CYGWIN__ */
