@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/v2.4/RCS/errors.h 2.4 1998/05/31 15:43:04 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.h 2.5 2000/05/14 13:51:53 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.2.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Error logging header file.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98 Andrew M. Bishop
+  This file Copyright 1996,97,98,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -29,6 +29,11 @@ ErrorLevel;
 
 /* In errors.c */
 
+/*+ The level of error logging +*/
+extern ErrorLevel LogLevel,     /*+ in the config file for syslog and stderr. +*/
+                  DebugLevel;   /*+ on the command line for stderr. +*/
+
+
 void  InitErrorHandler(char *name,int syslogable,int stderrable);
 char *PrintMessage(ErrorLevel errlev,const char* fmt, ...);
 
