@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/ftp.c 1.33 1999/09/11 14:00:02 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/ftp.c 1.47 2001/03/17 21:07:32 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6b.
   Functions for getting URLs using FTP.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -41,6 +41,11 @@ static char *proxy=NULL;
 static int server_ctrl=-1,      /*+ for the control connection to the server. +*/
            server_data=-1;      /*+ for the data connection to the server. +*/
 
+#if USE_ZLIB
+/*+ A flag to indicate that a compressed page was requested. +*/
+static int request_compressed=0;
+#endif
+
 /*+ A buffer to contain the reply +*/
 static char *bufferhead=NULL,   /*+ head. +*/
             *buffer=NULL,       /*+ body. +*/
@@ -74,7 +79,7 @@ char *FTP_Open(URL *Url)
 
  /* Sort out the host. */
 
- proxy=WhichProxy(Url->proto,Url->host);
+ proxy=ConfigStringURL(Proxies,Url);
  if(IsLocalNetHost(Url->host))
     proxy=NULL;
 
@@ -98,7 +103,7 @@ char *FTP_Open(URL *Url)
 
  /* Open the connection. */
 
- server_ctrl=OpenClientSocket(server_host,server_port,ConnectTimeout);
+ server_ctrl=OpenClientSocket(server_host,server_port);
  init_buffer(server_ctrl);
 
  if(server_ctrl==-1)
@@ -146,16 +151,30 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
    {
     char *head;
 
-    MakeRequestAuthorised(proxy,request_head);
+    /* Make the request OK for a proxy. */
+
+    MakeRequestProxyAuthorised(proxy,request_head);
+
+    /* Add the compression header */
+
+#if USE_ZLIB
+    if(ConfigBooleanURL(RequestCompressedData,Url) && !NotCompressed(NULL,Url->path))
+      {
+       request_compressed=1;
+       AddToHeader(request_head,"Accept-Encoding","deflate; q=1.0, x-gzip; q=0.9, gzip; q=0.9, identity; q=0.1");
+      }
+#endif
+
+    /* Send the request. */
 
     head=HeaderString(request_head);
 
     PrintMessage(ExtraDebug,"Outgoing Request Head (to proxy)\n%s",head);
 
     if(write_string(server_ctrl,head)==-1)
-       msg=PrintMessage(Warning,"Failed to write to remote FTP proxy [%!s].");
+       msg=PrintMessage(Warning,"Failed to write to remote FTP proxy; [%!s].");
     if(request_body && write_data(server_ctrl,request_body->content,request_body->length)==-1)
-       msg=PrintMessage(Warning,"Failed to write to remote FTP proxy [%!s].");
+       msg=PrintMessage(Warning,"Failed to write to remote FTP proxy; [%!s].");
 
     free(head);
 
@@ -164,7 +183,7 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  /* Else Sort out the path. */
 
- path=URLDecode(Url->path,0);
+ path=URLDecodeGeneric(Url->path);
  if(path[strlen(path)-1]=='/')
    {
     path[strlen(path)-1]=0;
@@ -185,7 +204,15 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
     pass=Url->pass;
    }
  else
-    WhatFTPUserPass(Url->host,&user,&pass);
+   {
+    user=ConfigStringURL(FTPAuthUser,Url);
+    if(!user)
+       user=ConfigString(FTPUserName);
+
+    pass=ConfigStringURL(FTPAuthPass,Url);
+    if(!pass)
+       pass=ConfigString(FTPPassWord);
+   }
 
  /* send all the RFC959 commands. */
 
@@ -193,7 +220,7 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: connected; got: %s",str);
 
     if(!file && !*path && str && isdigit(str[0]) && isdigit(str[1]) && isdigit(str[2]) && str[3]=='-')
@@ -229,13 +256,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  if(write_formatted(server_ctrl,"USER %s\r\n",user)==-1)
    {
-    msg=PrintMessage(Warning,"Failed to write 'USER' command to remote FTP host [%!s].");
+    msg=PrintMessage(Warning,"Failed to write 'USER' command to remote FTP host; [%!s].");
     return(msg);
    }
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: sent 'USER %s'; got: %s",user,str);
    }
  while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -257,13 +284,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
    {
     if(write_formatted(server_ctrl,"PASS %s\r\n",pass)==-1)
       {
-       msg=PrintMessage(Warning,"Failed to write 'PASS' command to remote FTP host [%!s].");
+       msg=PrintMessage(Warning,"Failed to write 'PASS' command to remote FTP host; [%!s].");
        return(msg);
       }
 
     do
       {
-       str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+       str=read_line_or_timeout(server_ctrl,str);
        PrintMessage(ExtraDebug,"FTP: sent 'PASS %s'; got: %s",pass,str);
 
        if(!file && !*path && str && isdigit(str[0]) && isdigit(str[1]) && isdigit(str[2]) && str[3]=='-')
@@ -300,13 +327,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  if(write_formatted(server_ctrl,"CWD %s\r\n",*path?path:"/")==-1)
    {
-    msg=PrintMessage(Warning,"Failed to write 'CWD' command to remote FTP host [%!s].");
+    msg=PrintMessage(Warning,"Failed to write 'CWD' command to remote FTP host; [%!s].");
     return(msg);
    }
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: sent 'CWD %s' got: %s",*path?path:"/",str);
 
     if(!file && str && isdigit(str[0]) && isdigit(str[1]) && isdigit(str[2]) && str[3]=='-')
@@ -344,18 +371,18 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
    {
     if(write_formatted(server_ctrl,"CWD %s\r\n",file)==-1)
       {
-       msg=PrintMessage(Warning,"Failed to write 'CWD' command to remote FTP host [%!s].");
+       msg=PrintMessage(Warning,"Failed to write 'CWD' command to remote FTP host; [%!s].");
        return(msg);
       }
 
     do
       {
-       str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+       str=read_line_or_timeout(server_ctrl,str);
        PrintMessage(ExtraDebug,"FTP: sent 'CWD %s' got: %s",file,str);
       }
     while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
 
-    if(!str || (atoi(str)!=250 && atoi(str)!=550 && atoi(str)!=501))
+    if(!str || (atoi(str)!=250 && atoi(str)!=501 && atoi(str)!=530 && atoi(str)!=550))
       {
        if(str)
          {
@@ -393,13 +420,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  if(write_formatted(server_ctrl,"TYPE %c\r\n",file?'I':'A')==-1)
    {
-    msg=PrintMessage(Warning,"Failed to write 'TYPE' command to remote FTP host [%!s].");
+    msg=PrintMessage(Warning,"Failed to write 'TYPE' command to remote FTP host; [%!s].");
     return(msg);
    }
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: sent 'TYPE %c'; got: %s",file?'I':'A',str);
    }
  while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -425,47 +452,47 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
       {
        if(write_formatted(server_ctrl,"SIZE %s\r\n",file)==-1)
          {
-          msg=PrintMessage(Warning,"Failed to write 'SIZE' command to remote FTP host [%!s].");
+          msg=PrintMessage(Warning,"Failed to write 'SIZE' command to remote FTP host; [%!s].");
           return(msg);
          }
 
        do
          {
-          str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+          str=read_line_or_timeout(server_ctrl,str);
           PrintMessage(ExtraDebug,"FTP: sent 'SIZE %s'; got: %s",file,str);
          }
        while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
 
-       if (atoi(str) == 213)
-          if (str[4])
-             sprintf(sizebuf, "%ld", (long)atoi(str+4));
+       if(atoi(str)==213)
+          if(str[4])
+             sprintf(sizebuf,"%ld",(long)atoi(str+4));
 
        if(write_formatted(server_ctrl,"MDTM %s\r\n",file)==-1)
          {
-          msg=PrintMessage(Warning,"Failed to write 'MDTM' command to remote FTP host [%!s].");
+          msg=PrintMessage(Warning,"Failed to write 'MDTM' command to remote FTP host; [%!s].");
           return(msg);
          }
 
        do
          {
-          str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+          str=read_line_or_timeout(server_ctrl,str);
           PrintMessage(ExtraDebug,"FTP: sent 'MDTM %s'; got: %s",file,str);
          }
        while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
 
-       if (atoi(str) == 213)
+       if(atoi(str)==213)
          {
-          int year, mon, mday, hour, min, sec;
+          int year,mon,mday,hour,min,sec;
 
-          if (sscanf(str, "%*d %4d%2d%2d%2d%2d%2d", &year, &mon, &mday, &hour, &min, &sec) == 6)
+          if(sscanf(str,"%*d %4d%2d%2d%2d%2d%2d",&year,&mon,&mday,&hour,&min,&sec)==6)
             {
-             memset(&modtime, 0, sizeof(modtime));
-             modtime.tm_year = year - 1900;
-             modtime.tm_mon = mon - 1;
-             modtime.tm_mday = mday;
-             modtime.tm_hour = hour;
-             modtime.tm_min = min;
-             modtime.tm_sec = sec;
+             memset(&modtime,0,sizeof(modtime));
+             modtime.tm_year=year-1900;
+             modtime.tm_mon=mon-1;
+             modtime.tm_mday=mday;
+             modtime.tm_hour=hour;
+             modtime.tm_min=min;
+             modtime.tm_sec=sec;
             }
          }
       }
@@ -475,13 +502,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  if(write_string(server_ctrl,"PASV\r\n")==-1)
    {
-    msg=PrintMessage(Warning,"Failed to write 'PASV' command to remote FTP host [%!s].");
+    msg=PrintMessage(Warning,"Failed to write 'PASV' command to remote FTP host; [%!s].");
     return(msg);
    }
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: sent 'PASV'; got: %s",str);
    }
  while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -518,7 +545,7 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
     if(host[l]==',')
        host[l]='.';
 
- server_data=OpenClientSocket(host,port_l+256*port_h,ConnectTimeout);
+ server_data=OpenClientSocket(host,port_l+256*port_h);
  init_buffer(server_data);
 
  if(server_data==-1)
@@ -531,17 +558,21 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
 
  if(!strcmp(request_head->method,"GET"))
    {
+    char *command;
+
     if(file)
       {
+       command="RETR";
+
        if(write_formatted(server_ctrl,"RETR %s\r\n",file)==-1)
          {
-          msg=PrintMessage(Warning,"Failed to write 'RETR' command to remote FTP host [%!s].");
+          msg=PrintMessage(Warning,"Failed to write 'RETR' command to remote FTP host; [%!s].");
           return(msg);
          }
 
        do
          {
-          str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+          str=read_line_or_timeout(server_ctrl,str);
           PrintMessage(ExtraDebug,"FTP: sent 'RETR %s'; got: %s",file,str);
          }
        while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -550,18 +581,37 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
       }
     else
       {
-       if(write_string(server_ctrl,"LIST\r\n")==-1)
-         {
-          msg=PrintMessage(Warning,"Failed to write 'LIST' command to remote FTP host [%!s].");
-          return(msg);
-         }
+       int i;
 
-       do
+       for(i=0;i<2;i++)
          {
-          str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
-          PrintMessage(ExtraDebug,"FTP: sent 'LIST'; got: %s",str);
+          if(i==0)
+             command="LIST -a";
+          else
+             command="LIST";
+
+          if(write_formatted(server_ctrl,"%s\r\n",command)==-1)
+            {
+             msg=PrintMessage(Warning,"Failed to write '%s' command to remote FTP host; [%!s].",command);
+             return(msg);
+            }
+
+          do
+            {
+             str=read_line_or_timeout(server_ctrl,str);
+             PrintMessage(ExtraDebug,"FTP: sent '%s'; got: %s",command,str);
+            }
+          while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
+
+          if(i==0 && str && (atoi(str)!=150 && atoi(str)!=125))
+            {
+             char *p=str+strlen(str)-1;
+             while(*p=='\n' || *p=='\r') *p--=0;
+             msg=PrintMessage(Warning,"Got '%s' message after sending '%s' command to FTP server.",str,command);
+            }
+          else
+             break;
          }
-       while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
 
        mimetype="text/html";
       }
@@ -579,10 +629,10 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
          {
           char *p=str+strlen(str)-1;
           while(*p=='\n' || *p=='\r') *p--=0;
-          msg=PrintMessage(Warning,"Got '%s' message after sending '%s' command to FTP server.",str,file?"RETR":"LIST");
+          msg=PrintMessage(Warning,"Got '%s' message after sending '%s' command to FTP server.",str,command);
          }
        else
-          msg=PrintMessage(Warning,"No reply from FTP server to '%s' command; timed out?",file?"RETR":"LIST");
+          msg=PrintMessage(Warning,"No reply from FTP server to '%s' command; timed out?",command);
        return(msg);
       }
    }
@@ -592,13 +642,13 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
       {
        if(write_formatted(server_ctrl,"STOR %s\r\n",file)==-1)
          {
-          msg=PrintMessage(Warning,"Failed to write 'STOR' command to remote FTP host [%!s].");
+          msg=PrintMessage(Warning,"Failed to write 'STOR' command to remote FTP host; [%!s].");
           return(msg);
          }
 
        do
          {
-          str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+          str=read_line_or_timeout(server_ctrl,str);
           PrintMessage(ExtraDebug,"FTP: sent 'STOR %s'; got: %s",file,str);
          }
        while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -673,12 +723,12 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
        if(msg_reply)
          {
           char *old_msg_reply=msg_reply;
-          msg_reply=HTMLString(msg_reply);
+          msg_reply=HTMLString(msg_reply,0);
           free(old_msg_reply);
          }
 
        buffer=HTMLMessageBody(-1,"FTPDir-Head",
-                              "url",URLDecode(Url->name,0),
+                              "url",Url->name,
                               "message",msg_reply,
                               NULL);
 
@@ -689,7 +739,7 @@ char *FTP_Request(URL *Url,Header *request_head,Body *request_body)
  else
    {
     buffer=HTMLMessageBody(-1,"FTPPut",
-                           "url",URLDecode(Url->name,0),
+                           "url",Url->name,
                            NULL);
 
    }
@@ -726,7 +776,37 @@ char *FTP_ReadHead(char *line)
  /* Take a simple route if it is proxied. */
 
  if(proxy)
-    return(read_line_or_timeout(server_ctrl,line,SocketTimeout));
+   {
+    char *newline;
+
+#if USE_ZLIB
+   repeat:
+#endif
+
+    newline=read_line_or_timeout(server_ctrl,line);
+
+#if USE_ZLIB
+    if(newline && request_compressed && !strncmp(newline,"Content-Encoding:",17))
+      {
+       int compression=WhichCompression(newline+17);
+
+       if(compression)
+         {
+          char *p=newline+17,*q=newline+strlen(newline)-1;
+          while(isspace(*p))
+             p++;
+          while(isspace(*q))
+             *q--=0;
+          PrintMessage(Inform,"Server has used 'Content-Encoding: %s'.",p); /* Used in audit-usage.pl */
+          init_zlib_buffer(server_ctrl,compression);
+          line=newline;
+          goto repeat;
+         }
+      }
+#endif
+
+    return(newline);
+   }
  
  /* Else send the header. */
 
@@ -765,7 +845,7 @@ int FTP_ReadBody(char *s,int n)
  /* Take a simple route if it is proxied. */
 
  if(proxy)
-    return(read_data_or_timeout(server_ctrl,s,n,SocketTimeout));
+    return(read_data_or_timeout(server_ctrl,s,n));
  
  /* Else send the data then the tail. */
 
@@ -778,7 +858,7 @@ int FTP_ReadBody(char *s,int n)
        s[m]=buffertail[nreadtail];
    }
  else if(!buffer && !buffertail) /* File not dir entry. */
-    m=read_data_or_timeout(server_data,s,n,SocketTimeout);
+    m=read_data_or_timeout(server_data,s,n);
  else if(buffer && buffertail)  /* Middle of dir entry */
    {
     for(;nread<nbuffer && m<n;nread++,m++)
@@ -824,7 +904,13 @@ int FTP_Close(void)
  /* Take a simple route if it is proxied. */
 
  if(proxy)
+   {
+#if USE_ZLIB
+    finish_zlib_buffer(server_ctrl);
+#endif
+
     return(CloseSocket(server_ctrl));
+   }
 
  /* Else say goodbye and close all of the sockets, */
 
@@ -835,7 +921,7 @@ int FTP_Close(void)
 
  do
    {
-    str=read_line_or_timeout(server_ctrl,str,SocketTimeout);
+    str=read_line_or_timeout(server_ctrl,str);
     PrintMessage(ExtraDebug,"FTP: sent 'QUIT'; got: %s",str);
    }
  while(str && (!isdigit(str[0]) || !isdigit(str[1]) || !isdigit(str[2]) || str[3]!=' '));
@@ -865,10 +951,10 @@ int FTP_Close(void)
 
 static char *htmlise_dir_entry(char *line)
 {
- int i,isdir=0;
+ int i,isdir=0,islink=0;
  char *q,*p[16],*l,*file=NULL,*link=NULL;
 
- l=read_line_or_timeout(server_data,line,SocketTimeout);
+ l=read_line_or_timeout(server_data,line);
 
  if(!l)
     return(l);
@@ -898,11 +984,14 @@ static char *htmlise_dir_entry(char *line)
 
     if(*p[0]=='d')
        isdir=1;
+
+    if(*p[0]=='l')
+       islink=1;
    }
 
  if(file)
    {
-    char *endf,endfc,*endl,endlc,*ll,*urlenc;
+    char *endf,endfc,*endl,endlc,*ll,*fileurlenc,*linkurlenc;
 
     endf=file;
     while(*endf && *endf!=' ' && *endf!='\t' && *endf!='\r' && *endf!='\n')
@@ -914,15 +1003,20 @@ static char *htmlise_dir_entry(char *line)
        endl++;
     endlc=*endl;
 
+    *endf=0;
+    fileurlenc=URLEncodePath(file);
+    *endf=endfc;
+
     *endl=0;
-    urlenc=URLEncode(link);
+    linkurlenc=URLEncodePath(link);
     *endl=endlc;
 
-    ll=(char*)malloc(strlen(l)+strlen(urlenc)+24);
+    ll=(char*)malloc(strlen(l)+strlen(fileurlenc)+strlen(linkurlenc)+40);
 
     strncpy(ll,l,file-l);
     strcpy(ll+(file-l),"<a href=\"");
-    strcat(ll,urlenc);
+    strcat(ll,"./");
+    strcat(ll,fileurlenc);
     if(isdir && *(endl-1)!='/')
        strcat(ll,"/");
     strcat(ll,"\">");
@@ -930,9 +1024,29 @@ static char *htmlise_dir_entry(char *line)
     strcat(ll,file);
     *endf=endfc;
     strcat(ll,"</a>");
-    strcat(ll,endf);
 
-    free(urlenc);
+    if(islink)
+      {
+       strncat(ll,endf,link-endf);
+       strcat(ll,"<a href=\"");
+       if(strncmp(linkurlenc,"../",3) && strncmp(linkurlenc,"./",2) && strncmp(linkurlenc,"/",1))
+          strcat(ll,"./");
+       strcat(ll,linkurlenc);
+       strcat(ll,"\">");
+       *endl=0;
+       strcat(ll,link);
+       *endl=endlc;
+       strcat(ll,"</a>");
+
+       strcat(ll,endl);
+      }
+    else
+      {
+       strcat(ll,endf);
+      }
+
+    free(fileurlenc);
+    free(linkurlenc);
     if(l)
        free(l);
     l=ll;
