@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/http.c 1.17 1999/10/10 12:19:46 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/http.c 1.23 2000/08/27 09:54:46 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Functions for getting URLs using HTTP.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -48,7 +48,7 @@ char *HTTP_Open(URL *Url)
  char *server_host=NULL;
  int server_port=Protocols[Protocol_HTTP].defport;
 
- proxy=WhichProxy(Url->proto,Url->host);
+ proxy=ConfigStringURL(Proxies,Url);
  if(IsLocalNetHost(Url->host))
     proxy=NULL;
 
@@ -72,7 +72,7 @@ char *HTTP_Open(URL *Url)
 
  /* Open the connection. */
 
- server=OpenClientSocket(server_host,server_port,ConnectTimeout);
+ server=OpenClientSocket(server_host,server_port);
  init_buffer(server);
 
  if(server==-1)
@@ -101,7 +101,7 @@ char *HTTP_Request(URL *Url,Header *request_head,Body *request_body)
  char *msg=NULL,*head;
 
  if(proxy)
-    MakeRequestAuthorised(proxy,request_head);
+    MakeRequestProxyAuthorised(proxy,request_head);
  else
     MakeRequestNonProxy(request_head);
 
@@ -113,13 +113,19 @@ char *HTTP_Request(URL *Url,Header *request_head,Body *request_body)
     PrintMessage(ExtraDebug,"Outgoing Request Head (to server)\n%s",head);
 
  if(write_string(server,head)==-1)
-    msg=PrintMessage(Warning,"Failed to write to remote HTTP %s [%!s].",proxy?"proxy":"server");
+    msg=PrintMessage(Warning,"Failed to write head to remote HTTP %s; [%!s].",proxy?"proxy":"server");
  if(request_body)
    {
+#if 0
+    if(request_body->content[request_body->length-1]!='\n')
+      {
+       request_body->content[request_body->length++]='\r';
+       request_body->content[request_body->length++]='\n';
+      }
+#endif
+
     if(write_data(server,request_body->content,request_body->length)==-1)
-       msg=PrintMessage(Warning,"Failed to write to remote HTTP %s [%!s].",proxy?"proxy":"server");
-    else if(request_body->content[request_body->length-1]!='\n')
-       write_string(server,"\r\n");
+       msg=PrintMessage(Warning,"Failed to write body to remote HTTP %s; [%!s].",proxy?"proxy":"server");
    }
 
  free(head);
@@ -138,7 +144,7 @@ char *HTTP_Request(URL *Url,Header *request_head,Body *request_body)
 
 char *HTTP_ReadHead(char *line)
 {
- return(read_line_or_timeout(server,line,SocketTimeout));
+ return(read_line_or_timeout(server,line));
 }
 
 
@@ -154,7 +160,7 @@ char *HTTP_ReadHead(char *line)
 
 int HTTP_ReadBody(char *s,int n)
 {
- return(read_data_or_timeout(server,s,n,SocketTimeout));
+ return(read_data_or_timeout(server,s,n));
 }
 
 
