@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/index.c 2.53 1998/11/24 17:43:17 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/index.c 2.71 2000/10/21 18:37:47 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.4.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Generate an index of the web pages that are cached in the system.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -35,7 +35,9 @@
 /*+ A type to contain the information required to sort the files. +*/
 typedef struct _FileIndex
 {
- char *name;                    /*+ The URL of the file. +*/
+ URL *url;                      /*+ The URL of the file. +*/
+ char *name;                    /*+ The name of the URL. +*/
+ char *host;                    /*+ The host of the URL. +*/
  char *type;                    /*+ The type (file extension) of the URL. +*/
  long time;                     /*+ The time of the file (access or modification as appropriate). +*/
 }
@@ -66,6 +68,9 @@ typedef enum _SortMode
 SortMode;
 
 
+/*+ The file descriptor of the sppol directory. +*/
+extern int fSpoolDir;
+
 /*+ The list of files. +*/
 static FileIndex **files=NULL;
 static int nfiles=0;
@@ -77,12 +82,14 @@ static time_t earliest=0;
 static time_t now;
 
 static void IndexRoot(int fd);
-static void IndexProtocol(int fd,char *proto,SortMode mode);
-static void IndexHost(int fd,char *proto,char *host,SortMode mode);
-static void IndexOutgoing(int fd,SortMode mode);
-static void IndexMonitor(int fd,SortMode mode);
-static void IndexLastTime(int fd,SortMode mode,char *name);
-static void IndexLatest(int fd,SortMode mode);
+static void IndexProtocol(int fd,char *proto,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+static void IndexHost(int fd,char *proto,char *host,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+static void IndexOutgoing(int fd,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+static void IndexMonitor(int fd,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+static void IndexLastTime(int fd,char *name,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+static void IndexLastOut(int fd,char *name,SortMode mode,int delopt,int refopt, int monopt,int allopt);
+
+static int is_indexed(URL *Url,ConfigItem config);
 
 static void add_dir(char *name,SortMode mode);
 static void add_file(char *name,SortMode mode);
@@ -107,24 +114,28 @@ void IndexPage(int fd,URL *Url)
 {
  char *newpath;
  char *proto=NULL,*host="";
- SortMode sort=NSortModes;
+ SortMode sort=NSortModes,s;
  Protocol *protocol=NULL;
- int outgoing=0,monitor=0,lasttime=0,prevtime=0,latest=0,main=0;
+ int outgoing=0,monitor=0,lasttime=0,prevtime=0,lastout=0,prevout=0,mainpage=0;
+ int delopt=0,refopt=0,monopt=0,allopt=0;
  int i;
 
  if(!strcmp(Url->path,"/index/url/"))
    {
-    URL *indexUrl;
-
-    if(Url->args && (indexUrl=SplitURL(Url->args)))
+    if(Url->args)
       {
+       char *url=URLDecodeFormArgs(Url->args);
+       URL *indexUrl=SplitURL(Url->args);
        char *localhost=GetLocalHost(1);
-       char *relocate=(char*)malloc(strlen(localhost)+strlen(indexUrl->proto)+strlen(indexUrl->host)+24);
-       sprintf(relocate,"http://%s/index/%s/%s?%s\r\n",localhost,indexUrl->proto,indexUrl->host,sorttype[Alpha]);
+       char *relocate=(char*)malloc(strlen(localhost)+strlen(indexUrl->proto)+strlen(indexUrl->host)+32);
+
+       sprintf(relocate,"http://%s/index/%s/%s?sort=%s\r\n",localhost,indexUrl->proto,indexUrl->host,sorttype[Alpha]);
        HTMLMessage(fd,301,"WWWOFFLE Index URL Redirect",relocate,"IndexURLRedirect",
-                   "url",Url->args,
+                   "url",url,
                    "link",relocate+7+strlen(localhost),
                    NULL);
+
+       free(url);
        FreeURL(indexUrl);
        free(localhost);
        free(relocate);
@@ -161,27 +172,55 @@ void IndexPage(int fd,URL *Url)
  monitor =!strcmp(proto,"monitor");
  lasttime=!strcmp(proto,"lasttime");
  prevtime=!strncmp(proto,"prevtime",8) && atoi(proto+8)>=0 && atoi(proto+8)<=NUM_PREVTIME_DIR;
- latest  =!strcmp(proto,"latest");
- main=(!protocol && !outgoing && !monitor && !lasttime && !prevtime && !latest);
+ lastout =!strcmp(proto,"lastout");
+ prevout =!strncmp(proto,"prevout",7) && atoi(proto+7)>=0 && atoi(proto+7)<=NUM_PREVTIME_DIR;
+ mainpage=(!protocol && !outgoing && !monitor && !lasttime && !prevtime && !lastout && !prevout);
 
  if(Url->args)
-    for(sort=0;sort<NSortModes;sort++)
-       if(!strcmp(Url->args,sorttype[sort]))
-          break;
+   {
+    char **args,**a;
+
+    args=SplitFormArgs(Url->args);
+
+    for(a=&args[0];*a;a++)
+      {
+       if(!strncmp(*a,"sort=",5))
+         {
+          for(s=0;s<NSortModes;s++)
+             if(!strcmp(*a+5,sorttype[s]))
+               {sort=s;break;}
+         }
+       else if(!strcmp(*a,"delete"))
+          delopt=1;
+       else if(!strcmp(*a,"refresh"))
+          refopt=1;
+       else if(!strcmp(*a,"monitor"))
+          monopt=1;
+       else if(!strcmp(*a,"all"))
+          allopt=1;
+      }
+
+    free(args[0]);
+    free(args);
+   }
+
+ files=NULL;
+ nfiles=0;
 
  now=time(NULL);
+ earliest=0;
 
  if((*host && (strchr(host,'/') || !strcmp(host,"..") || !strcmp(host,"."))) ||
-    (main && Url->path[7]) ||
-    ((outgoing || monitor || lasttime || prevtime || latest) && *host))
+    (mainpage && Url->path[7]) ||
+    ((outgoing || monitor || lasttime || prevtime || lastout || prevout || mainpage) && *host))
     HTMLMessage(fd,404,"WWWOFFLE Illegal Index Page",NULL,"IndexIllegal",
                 "url",Url->pathp,
                 NULL);
- else if(sort==NSortModes && (outgoing || monitor || lasttime || prevtime || protocol))
+ else if(sort==NSortModes)
    {
     char *localhost=GetLocalHost(1);
-    char *relocate=(char*)malloc(strlen(localhost)+strlen(Url->path)+strlen(sorttype[0])+12);
-    sprintf(relocate,"http://%s%s?%s\r\n",localhost,Url->path,sorttype[0]);
+    char *relocate=(char*)malloc(strlen(localhost)+strlen(Url->path)+strlen(sorttype[0])+16);
+    sprintf(relocate,"http://%s%s?sort=%s",localhost,Url->path,sorttype[0]);
     HTMLMessage(fd,301,"WWWOFFLE Index Redirect",relocate,"IndexRedirect",
                 "url",Url->pathp,
                 "link",relocate+7+strlen(localhost),
@@ -197,24 +236,35 @@ void IndexPage(int fd,URL *Url)
                     "type",prevtime?"prevtime":proto,
                     "subtype",prevtime?proto+8:host,
                     "sort",sorttype[sort],
+                    "delete" ,delopt?"&delete":"",
+                    "refresh",refopt?"&refresh":"",
+                    "monitor",monopt?"&monitor":"",
+                    "all"    ,allopt?"&all":"",
                     NULL);
 
     if(outgoing)
-       IndexOutgoing(fd,sort);
+       IndexOutgoing(fd,sort,delopt,refopt,monopt,allopt);
     else if(monitor)
-       IndexMonitor(fd,sort);
+       IndexMonitor(fd,sort,delopt,refopt,monopt,allopt);
     else if(lasttime || prevtime)
-       IndexLastTime(fd,sort,proto);
-    else if(latest)
-       IndexLatest(fd,sort);
+       IndexLastTime(fd,proto,sort,delopt,refopt,monopt,allopt);
+    else if(lastout || prevout)
+       IndexLastOut(fd,proto,sort,delopt,refopt,monopt,allopt);
     else if(protocol && !*host)
-       IndexProtocol(fd,proto,sort);
+       IndexProtocol(fd,proto,sort,delopt,refopt,monopt,allopt);
     else if(protocol && *host)
-       IndexHost(fd,proto,host,sort);
+       IndexHost(fd,proto,host,sort,delopt,refopt,monopt,allopt);
     else
        IndexRoot(fd);
 
     HTMLMessageBody(fd,"Index-Tail",
+                    "type",prevtime?"prevtime":proto,
+                    "subtype",prevtime?proto+8:host,
+                    "sort",sorttype[sort],
+                    "delete" ,delopt?"&delete":"",
+                    "refresh",refopt?"&refresh":"",
+                    "monitor",monopt?"&monitor":"",
+                    "all"    ,allopt?"&all":"",
                     NULL);
    }
 
@@ -242,9 +292,17 @@ static void IndexRoot(int fd)
   char *proto The protocol to index.
 
   SortMode mode The sort mode to use.
+
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexProtocol(int fd,char *proto,SortMode mode)
+static void IndexProtocol(int fd,char *proto,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
  DIR *dir;
  struct dirent* ent;
@@ -260,12 +318,12 @@ static void IndexProtocol(int fd,char *proto,SortMode mode)
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open spool directory '%s' [%!s]; index failed.",proto);chdir("..");
+   {PrintMessage(Warning,"Cannot open spool directory '%s' [%!s]; index failed.",proto);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read spool directory '%s' [%!s]; index failed.",proto);closedir(dir);chdir("..");
+   {PrintMessage(Warning,"Cannot read spool directory '%s' [%!s]; index failed.",proto);closedir(dir);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
  ent=readdir(dir);  /* skip .. */
 
@@ -276,7 +334,7 @@ static void IndexProtocol(int fd,char *proto,SortMode mode)
 
  closedir(dir);
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -295,30 +353,29 @@ static void IndexProtocol(int fd,char *proto,SortMode mode)
                  "proto",proto,
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    char *url=(char*)malloc(strlen(proto)+strlen(files[i]->name)+8);
-    URL *Url;
-
-    strcpy(url,proto);
-    strcat(url,"://");
-    strcat(url,files[i]->name);
-    Url=SplitURL(url);
-
     if(mode==Dated)
        dated_separator(fd,i,&lastdays,&lasthours);
 
     HTMLMessageBody(fd,"IndexProtocol-Body",
-                    "proto",Url->proto,
-                    "host",Url->host,
+                    "proto",proto,
+                    "host",files[i]->name,
                     "sort",sorttype[mode],
+                    "delete" ,delopt?"&delete":"",
+                    "refresh",refopt?"&refresh":"",
+                    "monitor",monopt?"&monitor":"",
+                    "all"    ,allopt?"&all":"",
                     NULL);
 
     free(files[i]->name);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nfiles);
@@ -330,6 +387,10 @@ static void IndexProtocol(int fd,char *proto,SortMode mode)
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -351,9 +412,17 @@ static void IndexProtocol(int fd,char *proto,SortMode mode)
   char *host The name of the subdirectory.
 
   SortMode mode The sort mode to use.
+
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexHost(int fd,char *proto,char *host,SortMode mode)
+static void IndexHost(int fd,char *proto,char *host,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
  DIR *dir;
  struct dirent* ent;
@@ -369,21 +438,37 @@ static void IndexHost(int fd,char *proto,char *host,SortMode mode)
 
  /* Open the spool subdirectory. */
 
+#if defined(__CYGWIN__)
+ if(strchr(host,':'))
+   {
+    char *colon=strchr(host,':');
+    *colon='!';
+   }
+#endif
+
  if(chdir(host))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s]; not indexed.",proto,host);chdir("..");
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s]; not indexed.",proto,host);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open directory '%s/%s' [%!s]; not indexed.",proto,host);chdir("../..");
+   {PrintMessage(Warning,"Cannot open directory '%s/%s' [%!s]; not indexed.",proto,host);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read directory '%s/%s' [%!s]; not indexed.",proto,host);closedir(dir);chdir("../..");
+   {PrintMessage(Warning,"Cannot read directory '%s/%s' [%!s]; not indexed.",proto,host);closedir(dir);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
  ent=readdir(dir);  /* skip .. */
 
+#if defined(__CYGWIN__)
+ if(strchr(host,'!'))
+   {
+    char *bang=strchr(host,'!');
+    *bang=':';
+   }
+#endif
+
  /* Add all of the file names. */
 
  while((ent=readdir(dir)))
@@ -391,7 +476,7 @@ static void IndexHost(int fd,char *proto,char *host,SortMode mode)
 
  closedir(dir);
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -411,33 +496,38 @@ static void IndexHost(int fd,char *proto,char *host,SortMode mode)
                  "host",host,
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    URL *Url=SplitURL(files[i]->name);
-
-    if(!IsNotIndexed(Url->proto,Url->host,Url->path,"host"))
+    if(allopt || is_indexed(files[i]->url,IndexListHost))
       {
-       char *decurl=URLDecode(Url->pathp,0);
+       char count[8];
+
+       sprintf(count,"%d",i);
 
        if(mode==Dated)
           dated_separator(fd,i,&lastdays,&lasthours);
 
        HTMLMessageBody(fd,"IndexHost-Body",
-                       "url",Url->name,
-                       "link",Url->link,
-                       "item",decurl,
+                       "count",count,
+                       "url",files[i]->url->name,
+                       "link",files[i]->url->link,
+                       "item",files[i]->url->pathp,
+                       "delete" ,delopt?"&delete":"",
+                       "refresh",refopt?"&refresh":"",
+                       "monitor",monopt?"&monitor":"",
                        NULL);
 
-       free(decurl);
-
        nindexed++;
       }
 
-    free(files[i]->name);
+    FreeURL(files[i]->url);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nindexed);
@@ -450,6 +540,10 @@ static void IndexHost(int fd,char *proto,char *host,SortMode mode)
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -467,9 +561,17 @@ static void IndexHost(int fd,char *proto,char *host,SortMode mode)
   int fd The file descriptor to write to.
 
   SortMode mode The method to use to sort the names.
+
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexOutgoing(int fd,SortMode mode)
+static void IndexOutgoing(int fd,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
  DIR *dir;
  struct dirent* ent;
@@ -485,12 +587,12 @@ static void IndexOutgoing(int fd,SortMode mode)
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open directory 'outgoing' [%!s]; not indexed.");chdir("..");
+   {PrintMessage(Warning,"Cannot open directory 'outgoing' [%!s]; not indexed.");fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read directory 'outgoing' [%!s]; not indexed.");closedir(dir);chdir("..");
+   {PrintMessage(Warning,"Cannot read directory 'outgoing' [%!s]; not indexed.");closedir(dir);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
  ent=readdir(dir);  /* skip .. */
 
@@ -501,7 +603,7 @@ static void IndexOutgoing(int fd,SortMode mode)
 
  closedir(dir);
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -521,33 +623,38 @@ static void IndexOutgoing(int fd,SortMode mode)
  HTMLMessageBody(fd,"IndexOutgoing-Head",
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    URL *Url=SplitURL(files[i]->name);
-
-    if(!IsNotIndexed(Url->proto,Url->host,Url->path,"outgoing"))
+    if(allopt || is_indexed(files[i]->url,IndexListOutgoing))
       {
-       char *decurl=URLDecode(Url->name,0);
+       char count[8];
+
+       sprintf(count,"%d",i);
 
        if(mode==Dated)
           dated_separator(fd,i,&lastdays,&lasthours);
 
        HTMLMessageBody(fd,"IndexOutgoing-Body",
-                       "url",Url->name,
-                       "link",Url->link,
-                       "item",decurl,
+                       "count",count,
+                       "url",files[i]->url->name,
+                       "link",files[i]->url->link,
+                       "item",files[i]->url->name,
+                       "delete" ,delopt?"&delete":"",
+                       "refresh",refopt?"&refresh":"",
+                       "monitor",monopt?"&monitor":"",
                        NULL);
 
-       free(decurl);
-
        nindexed++;
       }
 
-    free(files[i]->name);
+    FreeURL(files[i]->url);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nindexed);
@@ -558,6 +665,10 @@ static void IndexOutgoing(int fd,SortMode mode)
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -575,9 +686,17 @@ static void IndexOutgoing(int fd,SortMode mode)
   int fd The file descriptor to write to.
 
   SortMode mode The method to use to sort the names.
+
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexMonitor(int fd,SortMode mode)
+static void IndexMonitor(int fd,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
  DIR *dir;
  struct dirent* ent;
@@ -593,12 +712,12 @@ static void IndexMonitor(int fd,SortMode mode)
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open directory 'monitor' [%!s]; not indexed.");chdir("..");
+   {PrintMessage(Warning,"Cannot open directory 'monitor' [%!s]; not indexed.");fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read directory 'monitor' [%!s]; not indexed.");closedir(dir);chdir("..");
+   {PrintMessage(Warning,"Cannot read directory 'monitor' [%!s]; not indexed.");closedir(dir);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
  ent=readdir(dir);  /* skip .. */
 
@@ -609,7 +728,7 @@ static void IndexMonitor(int fd,SortMode mode)
 
  closedir(dir);
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -629,19 +748,23 @@ static void IndexMonitor(int fd,SortMode mode)
  HTMLMessageBody(fd,"IndexMonitor-Head",
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    URL *Url=SplitURL(files[i]->name);
-
-    if(!IsNotIndexed(Url->proto,Url->host,Url->path,"monitor"))
+    if(allopt || is_indexed(files[i]->url,IndexListMonitor))
       {
-       char *decurl=URLDecode(Url->name,0);
        int last,next;
        char laststr[8],nextstr[8];
+       char count[8];
 
-       MonitorTimes(Url,&last,&next);
+       sprintf(count,"%d",i);
+
+       MonitorTimes(files[i]->url,&last,&next);
 
        if(last>=(31*24))
           sprintf(laststr,"31+");
@@ -656,21 +779,22 @@ static void IndexMonitor(int fd,SortMode mode)
           dated_separator(fd,i,&lastdays,&lasthours);
 
        HTMLMessageBody(fd,"IndexMonitor-Body",
-                       "url",Url->name,
-                       "link",Url->link,
-                       "item",decurl,
+                       "count",count,
+                       "url",files[i]->url->name,
+                       "link",files[i]->url->link,
+                       "item",files[i]->url->name,
                        "last",laststr,
                        "next",nextstr,
+                       "delete" ,delopt?"&delete":"",
+                       "refresh",refopt?"&refresh":"",
+                       "monitor",monopt?"&monitor":"",
                        NULL);
 
-       free(decurl);
-
        nindexed++;
       }
 
-    free(files[i]->name);
+    FreeURL(files[i]->url);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nindexed);
@@ -681,6 +805,10 @@ static void IndexMonitor(int fd,SortMode mode)
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -697,12 +825,20 @@ static void IndexMonitor(int fd,SortMode mode)
 
   int fd The file descriptor to write to.
 
+  char *name The name of the directory.
+
   SortMode mode The method to use to sort the names.
 
-  char *name The name of the directory.
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexLastTime(int fd,SortMode mode,char *name)
+static void IndexLastTime(int fd,char *name,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
  DIR *dir;
  struct dirent* ent;
@@ -718,12 +854,12 @@ static void IndexLastTime(int fd,SortMode mode,char *name)
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not indexed.",name);chdir("..");
+   {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not indexed.",name);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not indexed.",name);closedir(dir);chdir("..");
+   {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not indexed.",name);closedir(dir);fchdir(fSpoolDir);
    HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
  ent=readdir(dir);  /* skip .. */
 
@@ -734,7 +870,7 @@ static void IndexLastTime(int fd,SortMode mode,char *name)
 
  closedir(dir);
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -755,33 +891,38 @@ static void IndexLastTime(int fd,SortMode mode,char *name)
                  "number",name+8,
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    URL *Url=SplitURL(files[i]->name);
-
-    if(!IsNotIndexed(Url->proto,Url->host,Url->path,"latest"))
+    if(allopt || is_indexed(files[i]->url,IndexListLatest))
       {
-       char *decurl=URLDecode(Url->name,0);
+       char count[8];
+
+       sprintf(count,"%d",i);
 
        if(mode==Dated)
           dated_separator(fd,i,&lastdays,&lasthours);
 
        HTMLMessageBody(fd,"IndexLastTime-Body",
-                       "url",Url->name,
-                       "link",Url->link,
-                       "item",decurl,
+                       "count",count,
+                       "url",files[i]->url->name,
+                       "link",files[i]->url->link,
+                       "item",files[i]->url->name,
+                       "delete" ,delopt?"&delete":"",
+                       "refresh",refopt?"&refresh":"",
+                       "monitor",monopt?"&monitor":"",
                        NULL);
 
-       free(decurl);
-
        nindexed++;
       }
 
-    free(files[i]->name);
+    FreeURL(files[i]->url);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nindexed);
@@ -793,6 +934,10 @@ static void IndexLastTime(int fd,SortMode mode,char *name)
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -805,89 +950,56 @@ static void IndexLastTime(int fd,SortMode mode,char *name)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Create an index of the latest pages.
+  Create an index of the pages that were in the outgoing index last time online.
 
   int fd The file descriptor to write to.
 
+  char *name The name of the directory.
+
   SortMode mode The method to use to sort the names.
+
+  int delopt The option to display the delete button in the index.
+
+  int refopt The option to display the refresh buttons in the index.
+
+  int monopt The option to display the monitor button in the index.
+
+  int allopt The option to show all pages including unlisted ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void IndexLatest(int fd,SortMode mode)
+static void IndexLastOut(int fd,char *name,SortMode mode,int delopt,int refopt, int monopt,int allopt)
 {
- int i,p,nindexed=0;
+ DIR *dir;
+ struct dirent* ent;
+ int i,nindexed=0;
  char total[8],indexed[8],notindexed[8];
  int lastdays=0,lasthours=0;
 
- earliest=now-IndexLatestDays*24*3600;
+ /* Open the lasttime subdirectory. */
 
- /* Add all of the files in the sub directories. */
+ if(chdir(name))
+   {PrintMessage(Warning,"Cannot change to directory '%s' [%!s]; not indexed.",name);
+   HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
- for(p=0;p<NProtocols;p++)
-   {
-    struct stat buf;
+ dir=opendir(".");
+ if(!dir)
+   {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not indexed.",name);fchdir(fSpoolDir);
+   HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
 
-    if(!lstat(Protocols[p].name,&buf) && S_ISDIR(buf.st_mode))
-      {
-       DIR *dir2;
-       struct dirent* ent2;
+ ent=readdir(dir);  /* skip .  */
+ if(!ent)
+   {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not indexed.",name);closedir(dir);fchdir(fSpoolDir);
+   HTMLMessageBody(fd,"IndexError-Body",NULL);return;}
+ ent=readdir(dir);  /* skip .. */
 
-       if(chdir(Protocols[p].name))
-         {PrintMessage(Warning,"Cannot change to directory '%s' [%!s]; not indexed.",Protocols[p].name);continue;}
+ /* Add all of the file names. */
 
-       /* Open the spool directory. */
+ while((ent=readdir(dir)))
+    add_file(ent->d_name,mode);
 
-       dir2=opendir(".");
-       if(!dir2)
-         {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not indexed.",Protocols[p].name);chdir("..");continue;}
+ closedir(dir);
 
-       ent2=readdir(dir2);  /* skip .  */
-       if(!ent2)
-         {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; index failed.",Protocols[p].name);closedir(dir2);chdir("..");continue;}
-       ent2=readdir(dir2);  /* skip .. */
-
-       /* Print all of the sub directories. */
-
-       while((ent2=readdir(dir2)))
-         {
-          struct stat buf2;
-
-          if(lstat(ent2->d_name,&buf2))
-             PrintMessage(Inform,"Cannot stat file '%s' [%!s]; race condition?",ent2->d_name);
-          else
-             if(S_ISDIR(buf2.st_mode) && buf2.st_mtime>earliest)
-               {
-                DIR *dir3;
-                struct dirent* ent3;
-                struct utimbuf utbuf;
-
-                if(chdir(ent2->d_name))
-                  {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s]; not indexed.",Protocols[p].name,ent2->d_name);continue;}
-
-                dir3=opendir(".");
-                if(!dir3)
-                  {PrintMessage(Warning,"Cannot open directory '%s/%s' [%!s]; not indexed.",Protocols[p].name,ent2->d_name);chdir("..");continue;}
-
-                ent3=readdir(dir3);  /* skip .  */
-                if(!ent3)
-                  {PrintMessage(Warning,"Cannot read directory '%s/%s' [%!s]; not indexed.",Protocols[p].name,ent2->d_name);closedir(dir3);chdir("..");continue;}
-                ent3=readdir(dir3);  /* skip .. */
-
-                while((ent3=readdir(dir3)))
-                   add_file(ent3->d_name,mode);
-
-                closedir(dir3);
-                chdir("..");
-
-                utbuf.actime=buf2.st_atime;
-                utbuf.modtime=buf2.st_mtime;
-                utime(ent2->d_name,&utbuf);
-               }
-         }
-
-       closedir(dir2);
-       chdir("..");
-      }
-   }
+ fchdir(fSpoolDir);
 
  /* Sort the files. */
 
@@ -904,46 +1016,57 @@ static void IndexLatest(int fd,SortMode mode)
 
  sprintf(total,"%d",nfiles);
 
- HTMLMessageBody(fd,"IndexLatest-Head",
+ HTMLMessageBody(fd,"IndexLastOut-Head",
+                 "number",name+7,
                  "total",total,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  for(i=0;i<nfiles;i++)
    {
-    URL *Url=SplitURL(files[i]->name);
-
-    if(!IsNotIndexed(Url->proto,Url->host,Url->path,"latest"))
+    if(allopt || is_indexed(files[i]->url,IndexListLatest))
       {
-       char *decurl=URLDecode(Url->name,0);
+       char count[8];
+
+       sprintf(count,"%d",i);
 
        if(mode==Dated)
           dated_separator(fd,i,&lastdays,&lasthours);
 
-       HTMLMessageBody(fd,"IndexLatest-Body",
-                       "url",Url->name,
-                       "link",Url->link,
-                       "item",decurl,
+       HTMLMessageBody(fd,"IndexLastOut-Body",
+                       "count",count,
+                       "url",files[i]->url->name,
+                       "link",files[i]->url->link,
+                       "item",files[i]->url->name,
+                       "delete" ,delopt?"&delete":"",
+                       "refresh",refopt?"&refresh":"",
+                       "monitor",monopt?"&monitor":"",
                        NULL);
 
-       free(decurl);
-
        nindexed++;
       }
 
-    free(files[i]->name);
+    FreeURL(files[i]->url);
     free(files[i]);
-    FreeURL(Url);
    }
 
  sprintf(indexed   ,"%d",nindexed);
  sprintf(notindexed,"%d",nfiles-nindexed);
 
- HTMLMessageBody(fd,"IndexLatest-Tail",
+ HTMLMessageBody(fd,"IndexLastOut-Tail",
+                 "number",name+7,
                  "total",total,
                  "indexed",indexed,
                  "notindexed",notindexed,
                  "sort",sorttype[mode],
+                 "delete" ,delopt?"&delete":"",
+                 "refresh",refopt?"&refresh":"",
+                 "monitor",monopt?"&monitor":"",
+                 "all"    ,allopt?"&all":"",
                  NULL);
 
  /* Tidy up and exit */
@@ -955,6 +1078,22 @@ static void IndexLatest(int fd,SortMode mode)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the specified URL is to be indexed.
+
+  int is_indexed Return true if it is to be indexed.
+
+  URL *Url The URL to check.
+
+  ConfigItem config The special configuration for type of index that is being checked.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int is_indexed(URL *Url,ConfigItem config)
+{
+ return(ConfigBooleanURL(config,Url) && ConfigBooleanURL(IndexListAny,Url));
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Add a directory to the list.
 
@@ -980,12 +1119,22 @@ static void add_dir(char *name,SortMode mode)
       }
     files[nfiles]=(FileIndex*)malloc(sizeof(FileIndex));
 
+#if defined(__CYGWIN__)
+    if(strchr(name,'!'))
+      {
+       char *bang=strchr(name,'!');
+       *bang=':';
+      }
+#endif
+
     files[nfiles]->name=(char*)malloc(strlen(name)+1);
     strcpy(files[nfiles]->name,name);
 
-    files[nfiles]->type="";
-
-    if(mode==MTime || mode==Dated)
+    if(mode==Domain)
+       files[nfiles]->host=files[nfiles]->name;
+    else if(mode==Type)
+       files[nfiles]->type="";
+    else if(mode==MTime || mode==Dated)
        files[nfiles]->time=buf.st_mtime;
     else if(mode==ATime)
        files[nfiles]->time=buf.st_atime;
@@ -1029,23 +1178,20 @@ static void add_file(char *name,SortMode mode)
       }
     files[nfiles]=(FileIndex*)malloc(sizeof(FileIndex));
 
-    files[nfiles]->name=url;
+    files[nfiles]->url=SplitURL(url);
+    files[nfiles]->name=files[nfiles]->url->name;
 
-    if(mode==Type)
+    if(mode==Domain)
+       files[nfiles]->host=files[nfiles]->url->host;
+    else if(mode==Type)
       {
-       char *p;
+       char *p=files[nfiles]->url->path+strlen(files[nfiles]->url->path);
 
-       files[nfiles]->type="";
-
-       p=strchr(files[nfiles]->name,'?');
-       if(!p)
-          p=files[nfiles]->name+strlen(files[nfiles]->name);
-
-       for(;p>url;p--)
+       for(;p>=files[nfiles]->url->path;p--)
           if(*p=='.')
             {files[nfiles]->type=p;break;}
           else if(*p=='/')
-             break;
+            {files[nfiles]->type="";break;}
       }
     else if(mode==MTime || mode==Dated)
        files[nfiles]->time=buf.st_mtime;
@@ -1149,28 +1295,18 @@ static int sort_type(FileIndex **a,FileIndex **b)
 
 static int sort_domain(FileIndex **a,FileIndex **b)
 {
- char *ap=strchr((*a)->name,'/');
- char *bp=strchr((*b)->name,'/');
+ char *ap=(*a)->host+strlen((*a)->host);
+ char *bp=(*b)->host+strlen((*b)->host);
  int chosen=0;
 
- if(ap)
-    ap=strchr(ap+2,'/');
- if(!ap)
-    ap=(*a)->name+strlen((*a)->name);
-
- if(bp)
-    bp=strchr(bp+2,'/');
- if(!bp)
-    bp=(*b)->name+strlen((*b)->name);
-
  do
    {
     ap--;
     bp--;
 
-    while(ap>(*a)->name && *ap!='.' && *ap!='/')
+    while(ap>(*a)->host && *ap!='.' && *ap!='/')
        ap--;
-    while(bp>(*b)->name && *bp!='.' && *bp!='/')
+    while(bp>(*b)->host && *bp!='.' && *bp!='/')
        bp--;
 
     chosen=strcmp(ap,bp);
@@ -1182,7 +1318,7 @@ static int sort_domain(FileIndex **a,FileIndex **b)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Used to sort the files into alphabetical order.
+  Used to sort the files into time order.
 
   int sort_time Returns the comparison of the times.
 
