@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/misc.c 2.36 1999/07/02 19:37:57 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/misc.c 2.49 2000/10/20 18:13:27 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Miscellaneous HTTP / HTML functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -44,7 +44,7 @@ URL *SplitURL(char *url)
  URL *Url=(URL*)malloc(sizeof(URL));
  char *copyurl,*mallocurl=malloc(strlen(url)+2);
  int i=0,n=0;
- char *colon,*slash,*at,*temppath,root[2];
+ char *colon,*slash,*at,*ques,*semi,*temppath,root[2];
 
  copyurl=mallocurl;
  strcpy(copyurl,url);
@@ -73,7 +73,8 @@ URL *SplitURL(char *url)
     slash=strchr(copyurl,'/');
    }
  else if(colon && !isdigit(*(colon+1)) &&
-         (!at || (slash && at>slash)))  /* http:www.foo.com/... */
+         (!slash || colon<slash) &&
+         (!at || (slash && at>slash)))  /* http:www.foo.com/...[:@]... */
    {
     *colon=0;
     Url->proto=(char*)malloc(colon-copyurl+1);
@@ -101,21 +102,31 @@ URL *SplitURL(char *url)
 
  if(at && at>copyurl && (!slash || slash>at))
    {
-    if(colon && at>colon)               /* user:pass@www.foo.com... */
+    char *at2;
+
+    if(colon && at>colon)               /* user:pass@www.foo.com...[/]... */
       {
        *colon=0;
-       Url->user=URLDecode(copyurl,0);
-       copyurl=colon+1;
+       Url->user=URLDecodeGeneric(copyurl);
        *at=0;
-       Url->pass=URLDecode(copyurl,0);
+       Url->pass=URLDecodeGeneric(colon+1);
        copyurl=at+1;
       }
-    else                                /* user@www.foo.com... */
+    else if(colon && (at2=strchr(at+1,'@')) && /* user@host:pass@www.foo.com...[/]... */
+            at2>colon && at2<slash)            /* [not actually valid, but common]    */
+      {
+       *colon=0;
+       Url->user=URLDecodeGeneric(copyurl);
+       *at2=0;
+       Url->pass=URLDecodeGeneric(colon+1);
+       copyurl=at2+1;
+      }
+    else                               /* user@www.foo.com...[:/]... */
       {
        *at=0;
-       Url->user=URLDecode(copyurl,0);
-       copyurl=at+1;
+       Url->user=URLDecodeGeneric(copyurl);
        Url->pass=NULL;
+       copyurl=at+1;
       }
    }
  else
@@ -127,6 +138,31 @@ URL *SplitURL(char *url)
     Url->pass=NULL;
    }
 
+ /* Parameters */
+
+ semi=strchr(copyurl,';');
+ ques=strchr(copyurl,'?');
+
+ Url->params=NULL;
+
+ if(semi && (!ques || semi<ques)) /* ../path;...[?]... */
+   {
+    *semi++=0;
+
+    if(*semi)
+       Url->params=URLRecodeFormArgs(semi);
+   }
+
+ /* Arguments */
+
+ if(ques)                       /* ../path?... */
+   {
+    *ques++=0;
+    Url->args=URLRecodeFormArgs(ques);
+   }
+ else
+    Url->args=NULL;
+
  /* Hostname */
 
  if(*copyurl=='/')              /* /path/... (local) */
@@ -139,29 +175,16 @@ URL *SplitURL(char *url)
     Url->host=copyurl;
     Url->local=0;
 
-    if(slash)                   /* www.foo.com/... */
+    if(slash && (!ques || slash<ques)) /* www.foo.com/...[?]... */
        copyurl=slash;
-    else                        /* www.foo.com */
+    else                        /* www.foo.com[?]... */
       {root[0]='/';root[1]=0;copyurl=root;}
    }
 
- /* Arguments */
-
- Url->args=NULL;
-
- for(i=0;copyurl[i];i++)
-    if(copyurl[i]=='?')
-      {
-       copyurl[i]=0;
-       Url->args=(char*)malloc(strlen(copyurl+i+1)+1);
-       strcpy(Url->args,copyurl+i+1);
-       break;
-      }
-
  /* Pathname */
 
- temppath=URLDecode(copyurl,0);
- Url->path=URLEncode(temppath);
+ temppath=URLDecodeGeneric(copyurl);
+ Url->path=URLEncodePath(temppath);
  free(temppath);
 
  /* Hostname (cont) */
@@ -190,7 +213,12 @@ URL *SplitURL(char *url)
 
  /* Canonicalise the URL. */
 
- Url->name=(char*)malloc(strlen(Url->proto)+strlen(Url->host)+strlen(Url->path)+(Url->args?strlen(Url->args):1)+8);
+ Url->name=(char*)malloc(strlen(Url->proto)+
+                         strlen(Url->host)+
+                         strlen(Url->path)+
+                         (Url->params?strlen(Url->params):0)+
+                         (Url->args?strlen(Url->args):0)+
+                         8);
 
  strcpy(Url->name,Url->proto);
  n=strlen(Url->proto);
@@ -207,6 +235,12 @@ URL *SplitURL(char *url)
  strcpy(Url->name+n,Url->path);
  n+=strlen(Url->path);
 
+ if(Url->params && *Url->params)
+   {
+    strcpy(Url->name+n,";");
+    strcpy(Url->name+n+1,Url->params);
+   }
+
  if(Url->args && *Url->args)
    {
     strcpy(Url->name+n,"?");
@@ -215,19 +249,32 @@ URL *SplitURL(char *url)
 
  if(Url->user)
    {
-    Url->file=(char*)malloc(strlen(Url->proto)+strlen(Url->hostp)+strlen(Url->user)+(Url->pass?strlen(Url->pass):0)+8);
+    char *encuserpass;
+
+    Url->file=(char*)malloc(strlen(Url->name)+
+                            3*strlen(Url->user)+
+                            (Url->pass?3*strlen(Url->pass):0)+
+                            8);
 
     n=Url->hostp-Url->name;
     strncpy(Url->file,Url->name,n);
 
-    strcpy(Url->file+n,Url->user);
-    n+=strlen(Url->user);
+    encuserpass=URLEncodePassword(Url->user);
+
+    strcpy(Url->file+n,encuserpass);
+    n+=strlen(encuserpass);
+
+    free(encuserpass);
 
     if(Url->pass)
       {
+       encuserpass=URLEncodePassword(Url->pass);
+
        strcpy(Url->file+n,":");
-       strcpy(Url->file+n+1,Url->pass);
-       n+=strlen(Url->pass)+1;
+       strcpy(Url->file+n+1,encuserpass);
+       n+=strlen(encuserpass)+1;
+
+       free(encuserpass);
       }
 
     strcpy(Url->file+n,"@");
@@ -279,6 +326,7 @@ URL *SplitURL(char *url)
 void AddURLPassword(URL *Url,char *user,char *pass)
 {
  int n;
+ char *encuserpass;
 
  Url->user=(char*)malloc(strlen(user)+1);
  strcpy(Url->user,user);
@@ -289,19 +337,29 @@ void AddURLPassword(URL *Url,char *user,char *pass)
     strcpy(Url->pass,pass);
    }
 
- Url->file=(char*)malloc(strlen(Url->name)+strlen(Url->user)+(Url->pass?strlen(Url->pass):0)+8);
+ Url->file=(char*)malloc(strlen(Url->name)+
+                         3*strlen(Url->user)+
+                         (Url->pass?3*strlen(Url->pass):0)+8);
 
  n=Url->hostp-Url->name;
  strncpy(Url->file,Url->name,n);
 
- strcpy(Url->file+n,Url->user);
- n+=strlen(Url->user);
+ encuserpass=URLEncodePassword(Url->user);
+
+ strcpy(Url->file+n,encuserpass);
+ n+=strlen(encuserpass);
+
+ free(encuserpass);
 
  if(Url->pass)
    {
+    encuserpass=URLEncodePassword(Url->pass);
+
     strcpy(Url->file+n,":");
-    strcpy(Url->file+n+1,Url->pass);
-    n+=strlen(Url->pass)+1;
+    strcpy(Url->file+n+1,encuserpass);
+    n+=strlen(encuserpass)+1;
+
+    free(encuserpass);
    }
 
  strcpy(Url->file+n,"@");
@@ -328,13 +386,14 @@ void FreeURL(URL *Url)
 
  free(Url->name);
 
- if(Url->proto) free(Url->proto);
- if(Url->host)  free(Url->host);
- if(Url->path)  free(Url->path);
- if(Url->args)  free(Url->args);
+ if(Url->proto)  free(Url->proto);
+ if(Url->host)   free(Url->host);
+ if(Url->path)   free(Url->path);
+ if(Url->params) free(Url->params);
+ if(Url->args)   free(Url->args);
 
- if(Url->user)  free(Url->user);
- if(Url->pass)  free(Url->pass);
+ if(Url->user)   free(Url->user);
+ if(Url->pass)   free(Url->pass);
 
  free(Url);
 }
@@ -441,11 +500,13 @@ char *CanonicaliseName(char *name)
 
  match=&name[strlen(name)-3];
  if(match>=name && !strcmp(match,"/.."))
+   {
     if(match==name)
        *++match=0;
     else
        while(match>name && *--match!='/')
           *match=0;
+   }
 
 #if 0 /* as used in cxref */
 
@@ -467,36 +528,49 @@ char *CanonicaliseName(char *name)
 }
 
 
+/* To understand why the URLDecode*() and URLEncode*() functions are coded this way see README.URL */
+
+/*+ For conversion from integer to hex string. +*/
+static char hexstring[16]="0123456789ABCDEF";
+
+/*+ For conversion from hex string to integer. +*/
+static char unhexstring[256]={ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x00-0x0f "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x10-0x1f "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x20-0x2f " !"#$%&'()*+,-./" */
+                               0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0, 0,  /* 0x30-0x3f "0123456789:;<=>?" */
+                               0,10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x40-0x4f "@ABCDEFGHIJKLMNO" */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x50-0x5f "PQRSTUVWXYZ[\]^_" */
+                               0,10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x60-0x6f "`abcdefghijklmno" */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x70-0x7f "pqrstuvwxyz{|}~ " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x80-0x8f "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0x90-0x9f "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0xa0-0xaf "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0xb0-0xbf "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0xc0-0xcf "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0xd0-0xdf "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0xe0-0xef "                " */
+                               0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; /* 0xf0-0xff "                " */
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Decode a string that has been UrlEncoded.
 
-  char *URLDecode Returns a malloced copy of the decoded string.
+  char *URLDecodeGeneric Returns a malloced copy of the decoded string.
 
-  const char *str The string to be decoded.
-
-  int isform Set to true if the string being decoded is from a form.
+  const char *str The generic string to be decoded.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *URLDecode(const char *str, int isform)
+char *URLDecodeGeneric(const char *str)
 {
  int i,j;
  char *copy=(char*)malloc(strlen(str)+1);
 
  for(i=0,j=0;str[i];i++)
-    if(str[i]=='+' && isform)
-       copy[j++]=' ';
-    else if(str[i]=='%')
+    if(str[i]=='%')
       {
        unsigned int val=0;
-       i++;
-       if(str[i]>='a') val=str[i]-'a'+10;
-       else if(str[i]>='A') val=str[i]-'A'+10;
-       else val=str[i]-'0';
-       val*=16;
-       i++;
-       if(str[i]>='a') val+=str[i]-'a'+10;
-       else if(str[i]>='A') val+=str[i]-'A'+10;
-       else val+=str[i]-'0';
+       val=16*unhexstring[(unsigned char)str[++i]];
+       val+=unhexstring[(unsigned char)str[++i]];
        copy[j++]=val;
       }
     else
@@ -509,38 +583,102 @@ char *URLDecode(const char *str, int isform)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Encode a string using the UrlEncode method as used with the POST method.
+  Decode a POSTed form or URL arguments that has been UrlEncoded.
 
-  char *UrlEncode Returns a malloced copy of the encoded string.
+  char *URLDecodeForm Returns a malloced copy of the decoded form data string.
 
-  const char *str The string to be encoded.
+  const char *str The form data string to be decoded.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *URLEncode(const char *str)
+char *URLDecodeFormArgs(const char *str)
+{
+ int i,j;
+ char *copy=(char*)malloc(strlen(str)+1);
+
+ /* This is the same as the generic function except that '+' is used for ' ' instead of '%20'. */
+
+ for(i=0,j=0;str[i];i++)
+    if(str[i]=='%')
+      {
+       unsigned int val=0;
+       val=16*unhexstring[(unsigned char)str[++i]];
+       val+=unhexstring[(unsigned char)str[++i]];
+       copy[j++]=val;
+      }
+    else if(str[i]=='+')
+       copy[j++]=' ';
+    else
+       copy[j++]=str[i];
+
+ copy[j]=0;
+
+ return(copy);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Take a POSTed form data or a URL argument string and URLDecode and URLEncode it again.
+
+  char *URLRecodeFormArgs Returns a malloced copy of the decoded and re-encoded form/argument string.
+
+  const char *str The form/argument string to be decoded and re-encoded.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *URLRecodeFormArgs(const char *str)
 {
  int i,j;
  char *copy=(char*)malloc(3*strlen(str)+1);
 
+ /*
+   The characters in the range 0x00-0x1f and 0x7f-0xff are always disallowed.
+   The '%' character is always disallowed because it is the quote character.
+   RFC 1738 section 2.2 calls " <>"#%{}|\^~[]`" unsafe characters, I make an exception for "|~".
+   RFC 1738 section 2.2 calls ";/?:@=&" reserved characters, I make an exception for "/:".
+   RFC 1866 section 8.2.1 says that ' ' is replaced by '+'.
+   I disallow "'" because it may lead to confusion.
+   The unencoded characters "&=;" on the input are left unencoded on the output
+   The encoded character "+" on the input is left encoded on the output
+ */
+
+ static char allowed[256]=
+ "                                "  /* 0x00-0x1f "                                " */
+ " !  $   ()* ,-./0123456789:     "  /* 0x20-0x3f " !"#$%&'()*+,-./0123456789:;<=>?" */
+ " ABCDEFGHIJKLMNOPQRSTUVWXYZ    _"  /* 0x40-0x5f "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_" */
+ " abcdefghijklmnopqrstuvwxyz | ~ "  /* 0x60-0x7f "`abcdefghijklmnopqrstuvwxyz{|}~ " */
+ "                                "  /* 0x80-0x9f "                                " */
+ "                                "  /* 0xa0-0xbf "                                " */
+ "                                "  /* 0xc0-0xdf "                                " */
+ "                                "; /* 0xe0-0xff "                                " */
+
  for(i=0,j=0;str[i];i++)
-    if(isalpha(str[i]) ||
-       isdigit(str[i]) ||
-       str[i]=='/' || str[i]=='?' || str[i]==':' || str[i]=='@' || str[i]=='=' ||
-       str[i]=='$' || str[i]=='-' || str[i]=='_' || str[i]=='.' || str[i]=='+' || str[i]=='!' ||
-       str[i]=='*' || str[i]=='(' || str[i]==')' || str[i]==',' || str[i]=='\'')
+    if(allowed[(unsigned char)str[i]]!=' ' || str[i]=='&' || str[i]=='=' || str[i]==';' || str[i]=='+')
        copy[j++]=str[i];
+    else if(str[i]=='%')
+      {
+       unsigned int val=0;
+       val=16*unhexstring[(unsigned char)str[++i]];
+       val+=unhexstring[(unsigned char)str[++i]];
+
+       if(allowed[val]!=' ')
+          copy[j++]=val;
+       else if(val==' ')
+          copy[j++]='+';
+       else
+         {
+          copy[j++]='%';
+          copy[j++]=toupper(str[i-1]);
+          copy[j++]=toupper(str[i]);
+         }
+      }
+    else if(str[i]==' ')
+       copy[j++]='+';
     else
       {
        unsigned int val=str[i];
        copy[j]='%';
-       if(val%16>9)
-          copy[j+2]=val%16+'A'-10;
-       else
-          copy[j+2]=val%16+'0';
-       val/=16;
-       if(val%16>9)
-          copy[j+1]=val%16+'A'-10;
-       else
-          copy[j+1]=val%16+'0';
+       copy[j+2]=hexstring[val%16];
+       val>>=4;
+       copy[j+1]=hexstring[val%16];
        j+=3;
       }
 
@@ -551,11 +689,114 @@ char *URLEncode(const char *str)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Encode a string using the UrlEncode method as used for passwords in URLs.
+  Encode a string suitable for use as a pathname.
 
-  char *UrlEncodePassword Returns a malloced copy of the encoded string.
+  char *URLEncodePath Returns a malloced copy of the encoded pathname string.
 
-  const char *str The string to be encoded.
+  const char *str The pathname string to be encoded.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *URLEncodePath(const char *str)
+{
+ int i,j;
+ char *copy=(char*)malloc(3*strlen(str)+1);
+
+ /*
+   The characters in the range 0x00-0x1f and 0x7f-0xff are always disallowed.
+   The '%' character is always disallowed because it is the quote character.
+   RFC 1738 section 2.2 calls " <>"#%{}|\^~[]`" unsafe characters, I make an exception for '~'.
+   RFC 1738 section 2.2 calls ";/?:@=&" reserved characters, I make an exception for "/:=".
+   I disallow "'" because it may lead to confusion.
+ */
+
+ static char allowed[256]=
+ "                                "  /* 0x00-0x1f "                                " */
+ " !  $   ()*+,-./0123456789:  =  "  /* 0x20-0x3f " !"#$%&'()*+,-./0123456789:;<=>?" */
+ " ABCDEFGHIJKLMNOPQRSTUVWXYZ    _"  /* 0x40-0x5f "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_" */
+ " abcdefghijklmnopqrstuvwxyz   ~ "  /* 0x60-0x7f "`abcdefghijklmnopqrstuvwxyz{|}~ " */
+ "                                "  /* 0x80-0x9f "                                " */
+ "                                "  /* 0xa0-0xbf "                                " */
+ "                                "  /* 0xc0-0xdf "                                " */
+ "                                "; /* 0xe0-0xff "                                " */
+
+ for(i=0,j=0;str[i];i++)
+    if(allowed[(unsigned char)str[i]]!=' ')
+       copy[j++]=str[i];
+    else
+      {
+       unsigned int val=str[i];
+       copy[j]='%';
+       copy[j+2]=hexstring[val%16];
+       val>>=4;
+       copy[j+1]=hexstring[val%16];
+       j+=3;
+      }
+
+ copy[j]=0;
+
+ return(copy);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Encode a string suitable for use with the POST method or as URL arguments.
+
+  char *URLEncodeFormArgs Returns a malloced copy of the encoded form data string.
+
+  const char *str The form data string to be encoded.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char *URLEncodeFormArgs(const char *str)
+{
+ int i,j;
+ char *copy=(char*)malloc(3*strlen(str)+1);
+
+ /*
+   The characters in the range 0x00-0x1f and 0x7f-0xff are always disallowed.
+   The '%' character is always disallowed because it is the quote character.
+   RFC 1738 section 2.2 calls " <>"#%{}|\^~[]`" unsafe characters, I make an exception for "|~".
+   RFC 1738 section 2.2 calls ";/?:@=&" reserved characters, I make an exception for "/:".
+   RFC 1866 section 8.2.1 says that ' ' is replaced by '+'.
+   I disallow ""'\`" because they may lead to confusion.
+ */
+
+ static char allowed[256]=
+ "                                "  /* 0x00-0x1f "                                " */
+ " !  $   ()* ,-./0123456789:     "  /* 0x20-0x3f " !"#$%&'()*+,-./0123456789:;<=>?" */
+ " ABCDEFGHIJKLMNOPQRSTUVWXYZ    _"  /* 0x40-0x5f "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_" */
+ " abcdefghijklmnopqrstuvwxyz | ~ "  /* 0x60-0x7f "`abcdefghijklmnopqrstuvwxyz{|}~ " */
+ "                                "  /* 0x80-0x9f "                                " */
+ "                                "  /* 0xa0-0xbf "                                " */
+ "                                "  /* 0xc0-0xdf "                                " */
+ "                                "; /* 0xe0-0xff "                                " */
+
+ for(i=0,j=0;str[i];i++)
+    if(allowed[(unsigned char)str[i]]!=' ')
+       copy[j++]=str[i];
+    else if(str[i]==' ')
+       copy[j++]='+';
+    else
+      {
+       unsigned int val=str[i];
+       copy[j]='%';
+       copy[j+2]=hexstring[val%16];
+       val>>=4;
+       copy[j+1]=hexstring[val%16];
+       j+=3;
+      }
+
+ copy[j]=0;
+
+ return(copy);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Encode a string suitable for use as a username / password in a URL.
+
+  char *URLEncodePassword Returns a malloced copy of the encoded username / password string.
+
+  const char *str The password / username string to be encoded.
   ++++++++++++++++++++++++++++++++++++++*/
 
 char *URLEncodePassword(const char *str)
@@ -563,26 +804,35 @@ char *URLEncodePassword(const char *str)
  int i,j;
  char *copy=(char*)malloc(3*strlen(str)+1);
 
+ /*
+   The characters in the range 0x00-0x1f and 0x7f-0xff are always disallowed.
+   The '%' character is always disallowed because it is the quote character.
+   RFC 1738 section 2.2 calls " <>"#%{}|\^~[]`" unsafe characters, I make an exception for '~'.
+   RFC 1738 section 2.2 calls ";/?:@=&" reserved characters, I make an exception for "=".
+   RFC 1738 section 3.1 says that "@:/" are disallowed.
+   I disallow "'()" because they may lead to confusion.
+ */
+
+ static char allowed[256]=
+ "                                "  /* 0x00-0x1f "                                " */
+ " !  $     *+,-. 0123456789   =  "  /* 0x20-0x3f " !"#$%&'()*+,-./0123456789:;<=>?" */
+ " ABCDEFGHIJKLMNOPQRSTUVWXYZ    _"  /* 0x40-0x5f "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_" */
+ " abcdefghijklmnopqrstuvwxyz   ~ "  /* 0x60-0x7f "`abcdefghijklmnopqrstuvwxyz{|}~ " */
+ "                                "  /* 0x80-0x9f "                                " */
+ "                                "  /* 0xa0-0xbf "                                " */
+ "                                "  /* 0xc0-0xdf "                                " */
+ "                                "; /* 0xe0-0xff "                                " */
+
  for(i=0,j=0;str[i];i++)
-    if(isalpha(str[i]) ||
-       isdigit(str[i]) ||
-       str[i]=='?' || str[i]==':' || str[i]=='=' ||
-       str[i]=='$' || str[i]=='-' || str[i]=='_' || str[i]=='.' || str[i]=='+' || str[i]=='!' ||
-       str[i]=='*' || str[i]=='(' || str[i]==')' || str[i]==',' || str[i]=='\'')
+    if(allowed[(unsigned char)str[i]]!=' ')
        copy[j++]=str[i];
     else
       {
        unsigned int val=str[i];
        copy[j]='%';
-       if(val%16>9)
-          copy[j+2]=val%16+'A'-10;
-       else
-          copy[j+2]=val%16+'0';
-       val/=16;
-       if(val%16>9)
-          copy[j+1]=val%16+'A'-10;
-       else
-          copy[j+1]=val%16+'0';
+       copy[j+2]=hexstring[val%16];
+       val>>=4;
+       copy[j+1]=hexstring[val%16];
        j+=3;
       }
 
@@ -592,6 +842,42 @@ char *URLEncodePassword(const char *str)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Split a form body or URL arguments up into the component parts.
+
+  char **SplitFormArgs Returns an array of pointers into a copy of the string.
+
+  char *str The form data or arguments to split.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char **SplitFormArgs(char *str)
+{
+ char **args=(char**)malloc(9*sizeof(char*));
+ char *p;
+ int i=0;
+
+ args[0]=(char*)malloc(strlen(str)+1);
+ strcpy(args[0],str);
+
+ p=args[0];
+ while(*p)
+   {
+    if(*p=='&' || *p==';')
+      {
+       *p=0;
+       if(i && !(i%8))
+          args=(char**)realloc((void*)args,(i+9)*sizeof(char*));
+       args[++i]=p+1;
+      }
+    p++;
+   }
+
+ args[++i]=NULL;
+
+ return(args);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Generate a hash value for a string.
 
@@ -629,6 +915,11 @@ char *MakeHash(const char *args)
 }
 
 
+/*+ Conversion from time_t to date string and back +*/
+static char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
+static char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Convert the time into an RFC 822 compliant date.
 
@@ -641,33 +932,42 @@ char *MakeHash(const char *args)
 
 char *RFC822Date(long t,int utc)
 {
- static char *week[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
- static char *month[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
  static char value[2][32];
  static int which=0;
+ char weekday[4];
+ char month[4];
  struct tm *tim;
 
- if(utc)
+ if(utc) /* Get UTC using English language. */
+   {
     tim=gmtime(&t);
- else
+
+    strcpy(weekday,weekdays[tim->tm_wday]);
+    strcpy(month,months[tim->tm_mon]);
+   }
+ else /* Get the local time using current language. */
    {
     tim=localtime(&t);
     if(tim->tm_isdst<0)
       {tim=gmtime(&t);utc=1;}
+
+    strftime(weekday,4,"%a",tim);
+    strftime(month,4,"%b",tim);
    }
 
  which^=1;
 
  /* Sun, 06 Nov 1994 08:49:37 GMT    ; RFC 822, updated by RFC 1123 */
+
  sprintf(value[which],"%3s, %02d %3s %4d %02d:%02d:%02d %s",
-         week[tim->tm_wday],
+         weekday,
          tim->tm_mday,
-         month[tim->tm_mon],
+         month,
          tim->tm_year+1900,
          tim->tm_hour,
          tim->tm_min,
          tim->tm_sec,
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
          utc?"GMT":tim->tm_zone
 #elif defined(__CYGWIN__)
          utc?"GMT":_tzname[tim->tm_isdst>0]
@@ -690,7 +990,6 @@ char *RFC822Date(long t,int utc)
 
 long DateToTimeT(const char *date)
 {
- static char *month[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
  int  year,day,hour,min,sec;
  char monthstr[16];
  long retval=0;
@@ -703,7 +1002,7 @@ long DateToTimeT(const char *date)
     int mon;
 
     for(mon=0;mon<12;mon++)
-       if(!strcmp(monthstr,month[mon]))
+       if(!strcmp(monthstr,months[mon]))
           break;
 
     tim.tm_sec=sec;
@@ -826,9 +1125,11 @@ char *Base64Encode(const char *str,int l)
   char* HTMLString Returns a safe HTML string.
 
   const char* c A non-safe HTML string.
+
+  int nbsp Use a non-breaking space in place of normal ones.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char* HTMLString(const char* c)
+char* HTMLString(const char* c,int nbsp)
 {
  int i=0,j=0,len=256-5;              /* 5 is the longest possible inserted amount */
  char* ret=(char*)malloc(257);
@@ -865,6 +1166,18 @@ char* HTMLString(const char* c)
           ret[j++]='p';
           ret[j++]=';';
           break;
+         case ' ':
+          if(nbsp)
+            {
+             ret[j++]='&';
+             ret[j++]='n';
+             ret[j++]='b';
+             ret[j++]='s';
+             ret[j++]='p';
+             ret[j++]=';';
+             break;
+            }
+         /* else fallthrough */
          default:
           ret[j++]=c[i];
          }
