@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/misc.h 2.20 1999/09/11 14:00:03 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/misc.h 2.28 2001/01/27 15:04:04 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6b.
   Miscellaneous HTTP / HTML functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -36,7 +36,8 @@ typedef struct _URL
  char *proto;                   /*+ The protocol. +*/
  char *host;                    /*+ The host. +*/
  char *path;                    /*+ The path. +*/
- char *args;                    /*+ The arguments. +*/
+ char *params;                  /*+ The parameters. +*/
+ char *args;                    /*+ The arguments (also known as the query in RFCs). +*/
 
  ProtocolP Protocol;            /*+ The protocol. +*/
 
@@ -93,27 +94,37 @@ char *MakeHash(const char *args);
 char *RFC822Date(long t,int utc);
 long DateToTimeT(const char *date);
 
-char *URLDecode(const char *str, int isform);
-char *URLEncode(const char *str);
+char *URLDecodeGeneric(const char *str);
+char *URLDecodeFormArgs(const char *str);
+
+char *URLRecodeFormArgs(const char *str);
+
+char *URLEncodePath(const char *str);
+char *URLEncodeFormArgs(const char *str);
 char *URLEncodePassword(const char *str);
 
+char **SplitFormArgs(char *str);
+
 char *Base64Decode(const char *str,int *l);
 char *Base64Encode(const char *str,int l);
 
-char* HTMLString(const char* c);
+char* HTMLString(const char* c,int nbsp);
 
 /* In io.c */
 
+void set_read_timeout(int timeout);
+
 char *fgets_realloc(char *buffer,FILE *file);
 
 void init_buffer(int fd);
+
 int empty_buffer(int fd);
 
 int read_data(int fd,char *buffer,int n);
-int read_data_or_timeout(int fd,char *buffer,int n,int timeout);
+int read_data_or_timeout(int fd,char *buffer,int n);
 
 char *read_line(int fd,char *line);
-char *read_line_or_timeout(int fd,char *line,int timeout);
+char *read_line_or_timeout(int fd,char *line);
 
 int write_data(int fd,const char *data,int n);
 
@@ -125,4 +136,10 @@ int write_formatted(int fd,const char *fmt,...) __attribute__ ((format (printf,2
 int write_formatted(int fd,const char *fmt,...);
 #endif
 
+#if USE_ZLIB
+int init_zlib_buffer(int fd,int direction);
+int finish_zlib_buffer(int fd);
+#endif
+
+
 #endif /* MISC_H */
