@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/purge.c 2.21 1999/08/18 18:46:30 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/purge.c 2.29 2000/10/21 18:46:00 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Purge old files from the cache.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98,99 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -25,10 +25,10 @@
 #include <dirent.h>
 #include <unistd.h>
 
-#if defined(__FreeBSD__) || defined(__NetBSD__)
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
-#elif defined(__sun__) && defined(__svr4__)
+#elif (defined(__sun__) && defined(__svr4__)) || defined(__irix__)
 #include <sys/statvfs.h>
 #elif defined(__osf__)
 #include <sys/mount.h>
@@ -36,7 +36,7 @@
 #include <sys/vfs.h>
 #endif
 
-#if defined(__sun__) && defined(__svr4__)
+#if (defined(__sun__) && defined(__svr4__)) || defined(__irix__)
 #define STATFS statvfs
 #else
 #define STATFS statfs
@@ -49,8 +49,14 @@
 #include "errors.h"
 
 
+/*+ The file descriptor of the sppol directory. +*/
+extern int fSpoolDir;
+
+
 /* Local functions */
-static long PurgeFiles(char *proto,char *host);
+static long PurgeFiles(char *proto,char *host,int def_age);
+static int what_purge_age(char *proto,char *host);
+static int what_purge_age_url(URL *Url);
 
 /*+ Set this to 0 for debugging so that nothing is deleted. +*/
 #define DO_DELETE 1
@@ -70,11 +76,16 @@ static int pass;
 /*+ The blocksize. +*/
 static long int blocksize;
 
+/*+ The configuraion file options, looked up once then used. +*/
+static int purge_use_mtime;
+static int purge_use_url;
+static int purge_default_age;
 
 /*
  Note: Since some UNIX versions (Solaris) have st_size/st_blocks different for
-       different files I have reverted to using 1+(st_size/blocksize) instead
-       of st_blocks for counting the number of blocks in a file.
+       different files I have reverted to using 1+(st_size/blocksze) instead
+       of st_blocks for counting the number of blocks in a file and using
+       st_size/blocksize for directories (since they are multiples of blocks).
        This gives a small error in some cases but is fast to calculate compared
        to the more accurate value of st_size/blocksize+!!(st_size%blocksize).
 */
@@ -93,13 +104,17 @@ void PurgeCache(int fd)
  struct STATFS sbuf;
  struct stat buf;
 
+ purge_default_age=what_purge_age("*","*");
+ purge_use_mtime=ConfigBoolean(PurgeUseMTime);
+ purge_use_url=ConfigBoolean(PurgeUseURL);
+
  now=time(NULL)+600;
 
  age_scale=-1;
 
- blocks_by_age=(int*)malloc((DefaultPurgeAge+2)*sizeof(int));
+ blocks_by_age=(int*)malloc((purge_default_age+2)*sizeof(int));
 
- for(i=0;i<=DefaultPurgeAge+1;i++)
+ for(i=0;i<=purge_default_age+1;i++)
     blocks_by_age[i]=0;
 
  if(stat(".",&buf) || buf.st_size==-1 || buf.st_blocks==-1)
@@ -107,24 +122,51 @@ void PurgeCache(int fd)
     PrintMessage(Warning,"Cannot determine the disk block size [%!s]; using 1024 instead.");
     blocksize=1024;
    }
+ else if(buf.st_blocks==0)
+   {
+    PrintMessage(Warning,"The number of blocks (0) looks wrong; using 1024 for blocksize.");
+    blocksize=1024;
+   }
  else
    {
     blocksize=buf.st_size/buf.st_blocks;
 
     if(blocksize!=512 && blocksize!=1024 && blocksize!=2048)
       {
-       PrintMessage(Warning,"The blocksize (%d) looks wrong; using 1024 instead.");
+       PrintMessage(Warning,"The blocksize (%d) looks wrong; using 1024 instead.",blocksize);
        blocksize=1024;
       }
    }
 
+ /* Note: blocksize can be only 512, 1024 or 2048. */
+
+ /* Handle this carefully to avoid overflows from doing blocks*blocksize/1024
+                                or any errors from doing (blocks/1024)*blocksize */
+#define Blocks_to_kB(blocks) \
+ (long)((blocksize==512)?((blocks)/2): \
+       ((blocksize==1024)?(blocks): \
+        (blocks)*2)) \
+
+
  if(STATFS(".",&sbuf) || sbuf.f_bsize==-1 || sbuf.f_bavail==-1)
    {
     PrintMessage(Warning,"Cannot determine the disk free space [%!s]; assuming 0.");
     diskfree=0;
    }
  else
-    diskfree=sbuf.f_bsize*sbuf.f_bavail/blocksize;
+   {
+    int bs=blocksize,dbs=sbuf.f_bsize;
+
+    /* Do this carefully to stop overflow and reduce errors. */
+
+    while(!(dbs&1) && !(bs&1))  /* remove powers of 2. */
+      {dbs>>=1;bs>>=1;}
+
+    /* If both were powers of 2 then there should be no problem (either dbs or bs is now 1).
+       If not then I am assuming that sbuf.f_bavail is larger than dbs so the error is smaller. */
+
+    diskfree=dbs*(sbuf.f_bavail/bs);
+   }
 
  for(pass=1;pass<=2;pass++)
    {
@@ -132,7 +174,7 @@ void PurgeCache(int fd)
 
     write_string(fd,"\n");
 
-    if(PurgeCacheSize)
+    if(ConfigInteger(PurgeCacheSize))
        if(pass==1)
           write_string(fd,"Pass 1: Checking dates and sizes of files:\n");
        else
@@ -142,12 +184,12 @@ void PurgeCache(int fd)
 
     if(pass==1)
       {
-       if(PurgeUseMTime)
+       if(ConfigBoolean(PurgeUseMTime))
           write_string(fd,"  (Using modification time.)\n");
        else
           write_string(fd,"  (Using last access time.)\n");
 
-       if(PurgeUseURL)
+       if(ConfigBoolean(PurgeUseURL))
           write_string(fd,"  (Using the full URL.)\n");
        else
           write_string(fd,"  (Using the hostname and protocol only.)\n");
@@ -170,18 +212,18 @@ void PurgeCache(int fd)
        dir_blocks=buf.st_size/blocksize;
 
        total_blocks+=dir_blocks;
-       blocks_by_age[DefaultPurgeAge+1]+=dir_blocks;
+       blocks_by_age[purge_default_age+1]+=dir_blocks;
 
        if(chdir(proto))
          {PrintMessage(Warning,"Cannot change to directory '%s' [%!s]; not purged.",proto);continue;}
 
        dir=opendir(".");
        if(!dir)
-         {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not purged.",proto);chdir("..");continue;}
+         {PrintMessage(Warning,"Cannot open directory '%s' [%!s]; not purged.",proto);fchdir(fSpoolDir);continue;}
 
        ent=readdir(dir);  /* skip .  */
        if(!ent)
-         {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not purged.",proto);closedir(dir);chdir("..");continue;}
+         {PrintMessage(Warning,"Cannot read directory '%s' [%!s]; not purged.",proto);closedir(dir);fchdir(fSpoolDir);continue;}
        ent=readdir(dir);  /* skip .. */
 
        /* Search through all of the sub directories. */
@@ -190,12 +232,16 @@ void PurgeCache(int fd)
          {
           struct stat buf;
 
-          if(lstat(ent->d_name,&buf))
+          if(stat(ent->d_name,&buf))
              PrintMessage(Inform,"Cannot stat directory '%s/%s' [%!s]; race condition?",proto,ent->d_name);
           else if(S_ISDIR(buf.st_mode))
             {
-             long file_blocks=PurgeFiles(proto,ent->d_name);
+             long file_blocks;
+             int age;
 
+             age=what_purge_age(proto,ent->d_name);
+
+             file_blocks=PurgeFiles(proto,ent->d_name,age);
              dir_blocks=buf.st_size/blocksize;
 
              if(file_blocks==-1)
@@ -215,31 +261,29 @@ void PurgeCache(int fd)
                 utbuf.modtime=buf.st_mtime;
                 utime(ent->d_name,&utbuf);
 
-                blocks_by_age[DefaultPurgeAge+1]+=dir_blocks;
+                blocks_by_age[purge_default_age+1]+=dir_blocks;
                 total_blocks+=file_blocks+dir_blocks;
                 total_dirs++;
                }
 
-             if(PurgeUseURL)
+             if(purge_use_url)
                {
                 if(file_blocks==-1)
                    write_formatted(fd,"Purged %6s://%-32s ; (empty) - deleted\n",proto,ent->d_name);
                 else
-                   write_formatted(fd,"Purged %6s://%-32s ; %5ld kB\n",proto,ent->d_name,(file_blocks+dir_blocks)*blocksize/1024);
+                   write_formatted(fd,"Purged %6s://%-32s ; %5ld kB\n",proto,ent->d_name,Blocks_to_kB(file_blocks+dir_blocks));
                }
              else
                {
-                int age=WhatPurgeAge(proto,ent->d_name,"/",NULL);
-
                 if(pass==2 && age>0)
                    age=(int)(age*age_scale+0.5);
 
                 if(age<0)
-                   write_formatted(fd,"Not Purged       %6s://%-32s ; %5ld kB\n",proto,ent->d_name,(file_blocks+dir_blocks)*blocksize/1024);
+                   write_formatted(fd,"Not Purged       %6s://%-32s ; %5ld kB\n",proto,ent->d_name,Blocks_to_kB(file_blocks+dir_blocks));
                 else if(file_blocks==-1)
                    write_formatted(fd,"Purged (%2d days) %6s://%-32s ; (empty) - deleted\n",age,proto,ent->d_name);
                 else
-                   write_formatted(fd,"Purged (%2d days) %6s://%-32s ; %5ld kB\n",age,proto,ent->d_name,(file_blocks+dir_blocks)*blocksize/1024);
+                   write_formatted(fd,"Purged (%2d days) %6s://%-32s ; %5ld kB\n",age,proto,ent->d_name,Blocks_to_kB(file_blocks+dir_blocks));
                }
             }
           else
@@ -257,62 +301,64 @@ void PurgeCache(int fd)
          }
 
        closedir(dir);
-       chdir("..");
+       fchdir(fSpoolDir);
       }
 
     write_string(fd,"\n");
-    write_formatted(fd,"Total of %d directories ; %ld kB\n",total_dirs,total_blocks*blocksize/1024);
+    write_formatted(fd,"Total of %d directories ; %ld kB\n",total_dirs,Blocks_to_kB(total_blocks));
 
     if(pass==1)
       {
        int age_for_size=-1,age_for_free=-1;
-       int age_blocks_used=blocks_by_age[DefaultPurgeAge+1];
-       int age_blocks_free=diskfree+total_blocks-blocks_by_age[DefaultPurgeAge+1];
+       int age_blocks_used=blocks_by_age[purge_default_age+1];
+       int age_blocks_free=diskfree+total_blocks-blocks_by_age[purge_default_age+1];
 
        write_string(fd,"\n");
        write_string(fd,"Age Profile of cached pages:\n");
-       write_string(fd,"  (All ages scaled to the range 0 -> default age.)\n");
+       write_formatted(fd,"  (All ages scaled to the range 0 -> %d (default age).)\n",purge_default_age);
        write_string(fd,"\n");
 
        write_formatted(fd,"Total not purged   ; %5ld kB (%6ld kB free)\n",
-                       age_blocks_used*blocksize/1024,age_blocks_free*blocksize/1024);
+                       Blocks_to_kB(age_blocks_used),Blocks_to_kB(age_blocks_free));
        write_string(fd,"\n");
 
-       for(i=0;i<=DefaultPurgeAge;i++)
+       for(i=0;i<=purge_default_age;i++)
          {
           age_blocks_used+=blocks_by_age[i];
           age_blocks_free-=blocks_by_age[i];
 
-          if(PurgeCacheSize && age_for_size<0 && (age_blocks_used*blocksize/1024)>(1024*PurgeCacheSize))
+          if(ConfigInteger(PurgeCacheSize) && age_for_size<0 &&
+             Blocks_to_kB(age_blocks_used)>(1024*ConfigInteger(PurgeCacheSize)))
             {
              age_for_size=i;
 
-             write_formatted(fd,"Cutoff Age is %2d days for %3d MB cache size\n",age_for_size,PurgeCacheSize);
+             write_formatted(fd,"Cutoff Age is %2d days for %3d MB cache size\n",age_for_size,ConfigInteger(PurgeCacheSize));
             }
 
-          if(PurgeDiskFree && diskfree && age_for_free<0 && (age_blocks_free*blocksize/1024)<(1024*PurgeDiskFree))
+          if(ConfigInteger(PurgeDiskFree) && diskfree && age_for_free<0 &&
+             Blocks_to_kB(age_blocks_free)<(1024*ConfigInteger(PurgeDiskFree)))
             {
              age_for_free=i;
 
-             write_formatted(fd,"Cutoff Age is %2d days for %3d MB disk free\n",age_for_free,PurgeDiskFree);
+             write_formatted(fd,"Cutoff Age is %2d days for %3d MB disk free\n",age_for_free,ConfigInteger(PurgeDiskFree));
             }
 
-          if(i==DefaultPurgeAge)
+          if(i==purge_default_age)
              write_formatted(fd,"Total all ages     ; %5ld kB (%6ld kB free)\n",
-                             (age_blocks_used-blocks_by_age[DefaultPurgeAge+1])*blocksize/1024,
-                             diskfree?age_blocks_free*blocksize/1024:0);
+                             Blocks_to_kB(age_blocks_used-blocks_by_age[purge_default_age+1]),
+                             diskfree?Blocks_to_kB(age_blocks_free):0);
           else
              write_formatted(fd,"Newer than %2d day%c ; %5ld kB (%6ld kB free)\n",i+1,i?'s':' ',
-                             (age_blocks_used-blocks_by_age[DefaultPurgeAge+1])*blocksize/1024,
-                             diskfree?age_blocks_free*blocksize/1024:0);
+                             Blocks_to_kB(age_blocks_used-blocks_by_age[purge_default_age+1]),
+                             diskfree?Blocks_to_kB(age_blocks_free):0);
          }
 
-       if(DefaultPurgeAge)
+       if(purge_default_age)
          {
           if(age_for_size!=-1 && (age_for_size<=age_for_free || age_for_free==-1))
-             age_scale=(double)age_for_size/(double)DefaultPurgeAge;
+             age_scale=(double)age_for_size/(double)purge_default_age;
           else if(age_for_free!=-1 && (age_for_free<age_for_size || age_for_size==-1))
-             age_scale=(double)age_for_free/(double)DefaultPurgeAge;
+             age_scale=(double)age_for_free/(double)purge_default_age;
          }
        else if(age_for_size!=-1 || age_for_free!=-1)
           age_scale=0;
@@ -368,7 +414,7 @@ void PurgeCache(int fd)
       }
    }
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  /* Purge the tmp.* files in temp. */
 
@@ -412,7 +458,7 @@ void PurgeCache(int fd)
       }
    }
 
- chdir("..");
+ fchdir(fSpoolDir);
 }
 
 
@@ -424,14 +470,15 @@ void PurgeCache(int fd)
   char *proto The name of the protocol directory to purge.
 
   char *host The name of the host directory to purge.
+
+  int def_age The default age to use for this host.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static long PurgeFiles(char *proto,char *host)
+static long PurgeFiles(char *proto,char *host,int def_age)
 {
  long blocks_left=-1;
  DIR *dir;
  struct dirent* ent;
- int def_age=WhatPurgeAge(proto,host,"/",NULL);
 
  /* Open the spool subdirectory. */
 
@@ -440,11 +487,11 @@ static long PurgeFiles(char *proto,char *host)
 
  dir=opendir(".");
  if(!dir)
-   {PrintMessage(Warning,"Cannot open directory '%s/%s' [%!s]; not purged.",proto,host);chdir("..");return(1);}
+   {PrintMessage(Warning,"Cannot open directory '%s/%s' [%!s]; not purged.",proto,host);fchdir(fSpoolDir);chdir(proto);return(1);}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read directory '%s/%s' [%!s]; not purged.",proto,host);closedir(dir);chdir("..");return(1);}
+   {PrintMessage(Warning,"Cannot read directory '%s/%s' [%!s]; not purged.",proto,host);closedir(dir);fchdir(fSpoolDir);chdir(proto);return(1);}
  ent=readdir(dir);  /* skip .. */
 
  /* Check all of the files for age, and delete as needed. */
@@ -460,7 +507,7 @@ static long PurgeFiles(char *proto,char *host)
 */
     else
       {
-       int age=DefaultPurgeAge;
+       int age=purge_default_age;
        time_t t=now;
 
        if(buf.st_mtime>now || buf.st_atime>now)
@@ -488,13 +535,13 @@ static long PurgeFiles(char *proto,char *host)
             }
           else if(*ent->d_name=='U')
              continue;
-          else if(PurgeUseURL)
+          else if(purge_use_url)
             {
              char *url=FileNameToURL(ent->d_name);
              if(url)
                {
                 URL *Url=SplitURL(url);
-                age=WhatPurgeAge(Url->proto,Url->host,Url->path,Url->args);
+                age=what_purge_age_url(Url);
                 FreeURL(Url);
                 free(url);
                }
@@ -504,7 +551,7 @@ static long PurgeFiles(char *proto,char *host)
           else
              age=def_age;
 
-          if(PurgeUseMTime)
+          if(purge_use_mtime)
              t=buf.st_mtime;
           else
              t=buf.st_atime;
@@ -530,14 +577,14 @@ static long PurgeFiles(char *proto,char *host)
             {
              int days=(now-t)/(24*3600);
 
-             days=days*DefaultPurgeAge/age; /* scale the age to fit into 0 -> DefaultPurgeAge */
+             days=days*purge_default_age/age; /* scale the age to fit into 0 -> DefaultPurgeAge */
 
-             if(days>DefaultPurgeAge)
-                days=DefaultPurgeAge;
+             if(days>purge_default_age)
+                days=purge_default_age;
              blocks_by_age[days]+=size;
             }
           else
-             blocks_by_age[DefaultPurgeAge+1]+=size;
+             blocks_by_age[purge_default_age+1]+=size;
          }
        else
          {
@@ -564,7 +611,66 @@ static long PurgeFiles(char *proto,char *host)
    }
 
  closedir(dir);
- chdir("..");
+ fchdir(fSpoolDir);
+ chdir(proto);
 
  return(blocks_left);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Determine the age to use when purging for a specified URL.
+
+  int what_purge_age Return the age in days.
+
+  char *proto The protocol to use.
+
+  char *host The host to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int what_purge_age(char *proto,char *host)
+{
+ int age;
+ URL Url;
+
+ /* cheat a bit here and poke in the values we need. */
+
+ Url.proto=proto;
+ Url.host=host;
+ Url.path="/";
+ Url.args=NULL;
+
+ age=ConfigIntegerURL(PurgeAges,&Url);
+
+ if(ConfigBoolean(PurgeDontGet) && ConfigBooleanMatchURL(DontGet,&Url))
+    age=0;
+
+ if(ConfigBoolean(PurgeDontCache) && (ConfigBooleanMatchURL(DontCache,&Url) || IsLocalNetHost(host)))
+    age=0;
+
+ return(age);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Determine the age to use when purging for a specified URL.
+
+  int what_purge_age_url Return the age in days.
+
+  URL *Url The URL that is to be purged.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int what_purge_age_url(URL *Url)
+{
+ int age;
+
+ age=ConfigIntegerURL(PurgeAges,Url);
+
+ if(ConfigBoolean(PurgeDontGet) && ConfigBooleanMatchURL(DontGet,Url))
+    age=0;
+
+ if(ConfigBoolean(PurgeDontCache) && (ConfigBooleanMatchURL(DontCache,Url) || IsLocalNetHost(Url->host)))
+    age=0;
+
+ return(age);
+}
