@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/sockets.h 1.2 1997/01/25 12:38:33 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/sockets.h 2.7 2000/08/27 09:54:47 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.0.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Socket function header file.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -19,8 +19,18 @@
 /* in sockets.c */
 
 int OpenClientSocket(char* host, int port);
+
 int OpenServerSocket(int port);
-int AcceptConnect(int s,char** name,int* port);
-void CloseSocket(int s);
+int AcceptConnect(int socket);
+
+int SocketRemoteName(int socket,char **name,char **ipname,int *port);
+int SocketLocalName(int socket,char **name,char **ipname,int *port);
+
+int CloseSocket(int socket);
+
+char *GetFQDN(void);
+
+void SetDNSTimeout(int timeout);
+void SetConnectTimeout(int timeout);
 
 #endif /* SOCKETS_H */
