@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/spool.c 2.49 1999/10/19 16:38:11 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/spool.c 2.56 2000/12/16 19:00:18 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   Handle all of the spooling of files in the spool directory.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98,99 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -38,6 +38,10 @@
 #endif
 
 
+/*+ The file descriptor of the sppol directory. +*/
+extern int fSpoolDir;
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Open a file in the outgoing directory to write into / read from.
 
@@ -56,10 +60,10 @@ int OpenOutgoingSpoolFile(int rw)
 
  /* Create the outgoing directory if needed and change to it */
 
- if(lstat("outgoing",&buf))
+ if(stat("outgoing",&buf))
    {
     PrintMessage(Inform,"Directory 'outgoing' does not exist [%!s]; creating one.");
-    if(mkdir("outgoing",DirPerm))
+    if(mkdir("outgoing",(mode_t)ConfigInteger(DirPerm)))
       {PrintMessage(Warning,"Cannot create directory 'outgoing' [%!s].");return(-1);}
    }
  else
@@ -77,11 +81,11 @@ int OpenOutgoingSpoolFile(int rw)
     DIR *dir=opendir(".");
 
     if(!dir)
-      {PrintMessage(Warning,"Cannot open current directory 'outgoing' [%!s].");chdir("..");return(-1);}
+      {PrintMessage(Warning,"Cannot open current directory 'outgoing' [%!s].");fchdir(fSpoolDir);return(-1);}
 
     ent=readdir(dir);  /* skip .  */
     if(!ent)
-      {PrintMessage(Warning,"Cannot read current directory 'outgoing' [%!s].");closedir(dir);chdir("..");return(-1);}
+      {PrintMessage(Warning,"Cannot read current directory 'outgoing' [%!s].");closedir(dir);fchdir(fSpoolDir);return(-1);}
     ent=readdir(dir);  /* skip .. */
 
     while((ent=readdir(dir)))
@@ -108,14 +112,14 @@ int OpenOutgoingSpoolFile(int rw)
    }
  else
    {
-    fd=open(name,O_WRONLY|O_CREAT|O_EXCL|O_BINARY,FilePerm);
+    fd=open(name,O_WRONLY|O_CREAT|O_EXCL|O_BINARY,(mode_t)ConfigInteger(FilePerm));
     if(fd==-1)
        PrintMessage(Warning,"Cannot open file 'outgoing/%s' [%!s]",name);
    }
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  return(fd);
 }
@@ -149,7 +153,7 @@ void CloseOutgoingSpoolFile(int fd,URL *Url)
  *newname='U';
 
  unlink(newname);
- ufd=open(newname,O_WRONLY|O_CREAT|O_EXCL|O_BINARY,FilePerm);
+ ufd=open(newname,O_WRONLY|O_CREAT|O_EXCL|O_BINARY,(mode_t)ConfigInteger(FilePerm));
  if(ufd!=-1)
    {
     write_string(ufd,Url->file);
@@ -162,7 +166,7 @@ void CloseOutgoingSpoolFile(int fd,URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(newname);
 }
@@ -180,7 +184,7 @@ int ExistsOutgoingSpoolFile(URL *Url)
 {
  struct stat buf;
  char *name;
- int exists=0;
+ int existsO,existsU;
 
  if(chdir("outgoing"))
    {PrintMessage(Warning,"Cannot change to directory 'outgoing' [%!s].");return(0);}
@@ -188,17 +192,20 @@ int ExistsOutgoingSpoolFile(URL *Url)
  /* Stat the outgoing file */
 
  name=URLToFileName(Url);
- *name='O';
 
- exists=!stat(name,&buf);
+ *name='O';
+ existsO=!stat(name,&buf);
+
+ *name='U';
+ existsU=!stat(name,&buf);
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(name);
 
- return(exists);
+ return(existsO&&existsU);
 }
 
 
@@ -225,7 +232,7 @@ char *HashOutgoingSpoolFile(URL *Url)
 
  fd=open(name,O_RDONLY|O_BINARY);
  if(fd==-1)
-   {PrintMessage(Warning,"Cannot open outgoing request to create hash [%!s].");free(name);chdir("..");return(NULL);}
+   {PrintMessage(Warning,"Cannot open outgoing request to create hash [%!s].");free(name);fchdir(fSpoolDir);return(NULL);}
 
  req=(char*)malloc(256+1);
 
@@ -241,7 +248,7 @@ char *HashOutgoingSpoolFile(URL *Url)
     while((rr=read(fd,&req[r],256))>0);
    }
  else if(r==-1)
-   {PrintMessage(Warning,"Cannot read from outgoing request to create hash [%!s].");close(fd);free(name);free(req);chdir("..");return(NULL);}
+   {PrintMessage(Warning,"Cannot read from outgoing request to create hash [%!s].");close(fd);free(name);free(req);fchdir(fSpoolDir);return(NULL);}
 
  req[r]=0;
 
@@ -251,7 +258,7 @@ char *HashOutgoingSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(name);
  free(req);
@@ -300,11 +307,11 @@ char *DeleteOutgoingSpoolFile(URL *Url)
     DIR *dir=opendir(".");
 
     if(!dir)
-      {err=PrintMessage(Warning,"Cannot open current directory 'outgoing' [%!s].");chdir("..");return(err);}
+      {err=PrintMessage(Warning,"Cannot open current directory 'outgoing' [%!s].");fchdir(fSpoolDir);return(err);}
 
     ent=readdir(dir);  /* skip .  */
     if(!ent)
-      {err=PrintMessage(Warning,"Cannot read current directory 'outgoing' [%!s].");closedir(dir);chdir("..");return(err);}
+      {err=PrintMessage(Warning,"Cannot read current directory 'outgoing' [%!s].");closedir(dir);fchdir(fSpoolDir);return(err);}
     ent=readdir(dir);  /* skip .. */
 
     while((ent=readdir(dir)))
@@ -318,7 +325,7 @@ char *DeleteOutgoingSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  return(err);
 }
@@ -342,10 +349,10 @@ int OpenWebpageSpoolFile(int rw,URL *Url)
 
  /* Create the spool directory if needed and change to it. */
 
- if(lstat(Url->proto,&buf))
+ if(stat(Url->proto,&buf))
    {
     PrintMessage(Inform,"Directory '%s' does not exist [%!s]; creating one.",Url->proto);
-    if(mkdir(Url->proto,DirPerm))
+    if(mkdir(Url->proto,(mode_t)ConfigInteger(DirPerm)))
       {PrintMessage(Warning,"Cannot create directory '%s' [%!s].",Url->proto);return(-1);}
    }
  else
@@ -355,18 +362,18 @@ int OpenWebpageSpoolFile(int rw,URL *Url)
  if(chdir(Url->proto))
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return(-1);}
 
- if(lstat(Url->dir,&buf))
+ if(stat(Url->dir,&buf))
    {
     PrintMessage(Inform,"Directory '%s/%s' does not exist [%!s]; creating one.",Url->proto,Url->dir);
-    if(mkdir(Url->dir,DirPerm))
-      {PrintMessage(Warning,"Cannot create directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return(-1);}
+    if(mkdir(Url->dir,(mode_t)ConfigInteger(DirPerm)))
+      {PrintMessage(Warning,"Cannot create directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return(-1);}
    }
  else
     if(!S_ISDIR(buf.st_mode))
-      {PrintMessage(Warning,"The file '%s/%s' is not a directory.",Url->proto,Url->dir);chdir("..");return(-1);}
+      {PrintMessage(Warning,"The file '%s/%s' is not a directory.",Url->proto,Url->dir);fchdir(fSpoolDir);return(-1);}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return(-1);}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return(-1);}
 
  /* Open the file for the web page. */
 
@@ -376,14 +383,14 @@ int OpenWebpageSpoolFile(int rw,URL *Url)
  if(rw)
     fd=open(file,O_RDONLY|O_BINARY);
  else
-    fd=open(file,O_RDWR|O_CREAT|O_BINARY,FilePerm);
+    fd=open(file,O_RDWR|O_CREAT|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
  if(!rw && fd!=-1)
    {
     int ufd;
 
     *file='U';
-    ufd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,FilePerm);
+    ufd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
     if(ufd!=-1)
       {
@@ -402,14 +409,15 @@ int OpenWebpageSpoolFile(int rw,URL *Url)
  if(!rw && fd!=-1)
     utime(".",NULL);
 
- chdir("..");
+ fchdir(fSpoolDir);
+ chdir(Url->proto);
 
  if(!rw && fd!=-1)
     utime(".",NULL);
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(file);
 
@@ -437,7 +445,7 @@ char *DeleteWebpageSpoolFile(URL *Url,int all)
    {err=PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return(err);}
 
  if(chdir(Url->dir))
-   {err=PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return(err);}
+   {err=PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return(err);}
 
  /* Delete the file for the web page. */
 
@@ -447,11 +455,11 @@ char *DeleteWebpageSpoolFile(URL *Url,int all)
     DIR *dir=opendir(".");
 
     if(!dir)
-      {err=PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("../..");return(err);}
+      {err=PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return(err);}
 
     ent=readdir(dir);  /* skip .  */
     if(!ent)
-      {err=PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url->proto,Url->dir);closedir(dir);chdir("../..");return(err);}
+      {err=PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url->proto,Url->dir);closedir(dir);fchdir(fSpoolDir);return(err);}
     ent=readdir(dir);  /* skip .. */
 
     while((ent=readdir(dir)))
@@ -464,10 +472,9 @@ char *DeleteWebpageSpoolFile(URL *Url,int all)
             {
              URL *delUrl=SplitURL(delurl);
 
-             chdir("../..");
+             fchdir(fSpoolDir);
 
-             if(ExistsLastTimeSpoolFile(delUrl))
-                DeleteLastTimeSpoolFile(delUrl);
+             DeleteLastTimeSpoolFile(delUrl);
 
              chdir(Url->proto);
              chdir(Url->dir);
@@ -483,12 +490,13 @@ char *DeleteWebpageSpoolFile(URL *Url,int all)
 
     closedir(dir);
 
-    chdir("..");
+    fchdir(fSpoolDir);
+    chdir(Url->proto);
 
     if(rmdir(Url->dir))
       err=PrintMessage(Warning,"Cannot delete what should be an empty directory '%s/%s' [%!s].",Url->proto,Url->dir);
 
-    chdir("..");
+    fchdir(fSpoolDir);
    }
  else
    {
@@ -519,10 +527,9 @@ char *DeleteWebpageSpoolFile(URL *Url,int all)
        utime(".",&utbuf);
       }
 
-    chdir("../..");
+    fchdir(fSpoolDir);
 
-    if(ExistsLastTimeSpoolFile(Url))
-       DeleteLastTimeSpoolFile(Url);
+    DeleteLastTimeSpoolFile(Url);
    }
 
  return(err);
@@ -547,7 +554,7 @@ void TouchWebpageSpoolFile(URL *Url,time_t when)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Touch the file for the web page. */
 
@@ -576,7 +583,7 @@ void TouchWebpageSpoolFile(URL *Url,time_t when)
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(file);
 }
@@ -594,6 +601,7 @@ time_t ExistsWebpageSpoolFile(URL *Url)
 {
  struct stat buf;
  char *file;
+ int existsD,existsU;
 
  /* Change to the spool directory. */
 
@@ -601,23 +609,28 @@ time_t ExistsWebpageSpoolFile(URL *Url)
     return(0);
 
  if(chdir(Url->dir))
-   {chdir("..");return(0);}
+   {fchdir(fSpoolDir);return(0);}
 
  /* Stat the file for the web page. */
 
  file=URLToFileName(Url);
- *file='D';
 
- if(stat(file,&buf))
-    buf.st_atime=0;
+ *file='U';
+ existsU=!stat(file,&buf);
+
+ *file='D';
+ existsD=!stat(file,&buf);
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(file);
 
- return(buf.st_atime);
+ if(existsU&&existsD)
+    return(buf.st_atime);
+ else
+    return(0);
 }
 
 
@@ -638,7 +651,7 @@ void CreateBackupWebpageSpoolFile(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Create the filenames and rename the files. */
 
@@ -663,7 +676,7 @@ void CreateBackupWebpageSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(bakfile);
  free(orgfile);
@@ -687,7 +700,7 @@ void RestoreBackupWebpageSpoolFile(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Create the filenames and rename the files. */
 
@@ -710,7 +723,7 @@ void RestoreBackupWebpageSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(bakfile);
  free(orgfile);
@@ -733,7 +746,7 @@ void DeleteBackupWebpageSpoolFile(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Delete the file for the backup web page. */
 
@@ -750,7 +763,7 @@ void DeleteBackupWebpageSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(bakfile);
 }
@@ -765,7 +778,12 @@ void DeleteBackupWebpageSpoolFile(URL *Url)
 void CreateLockWebpageSpoolFile(URL *Url)
 {
  struct stat buf;
- char *lokfile,*orgfile;
+ char *lockfile,*orgfile;
+
+ /* Check for configuration file option. */
+
+ if(!ConfigBoolean(LockFiles))
+    return;
 
  /* Change to the spool directory. */
 
@@ -773,31 +791,31 @@ void CreateLockWebpageSpoolFile(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Create the lock file for the web page. */
 
  orgfile=URLToFileName(Url);
 
- lokfile=(char*)malloc(strlen(orgfile)+2);
- strcpy(lokfile,orgfile);
+ lockfile=(char*)malloc(strlen(orgfile)+2);
+ strcpy(lockfile,orgfile);
 
- *lokfile='L';
+ *lockfile='L';
  *orgfile='D';
 
- if(!stat(lokfile,&buf))
+ if(!stat(lockfile,&buf))
     PrintMessage(Inform,"Lock file already exists for '%s'.",Url->name);
  else
    {
-    if(link(orgfile,lokfile))
+    if(link(orgfile,lockfile))
        PrintMessage(Warning,"Cannot make a lock file for '%s/%s/%s' [%!s].",Url->proto,Url->dir,orgfile);
    }
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
- free(lokfile);
+ free(lockfile);
  free(orgfile);
 }
 
@@ -814,28 +832,33 @@ int ExistsLockWebpageSpoolFile(URL *Url)
 {
  struct stat buf;
  char *file;
- int exists=0;
+ int existsL;
+
+ /* Check for configuration file option. */
+
+ if(!ConfigBoolean(LockFiles))
+    return(0);
 
  if(chdir(Url->proto))
     return(0);
 
  if(chdir(Url->dir))
-   {chdir("..");return(0);}
+   {fchdir(fSpoolDir);return(0);}
 
  /* Stat the file for the web page. */
 
  file=URLToFileName(Url);
- *file='L';
 
- exists=!stat(file,&buf);
+ *file='L';
+ existsL=!stat(file,&buf);
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
  free(file);
 
- return(exists);
+ return(existsL);
 }
 
 
@@ -847,7 +870,12 @@ int ExistsLockWebpageSpoolFile(URL *Url)
 
 void DeleteLockWebpageSpoolFile(URL *Url)
 {
- char *lokfile;
+ char *lockfile;
+
+ /* Check for configuration file option. */
+
+ if(!ConfigBoolean(LockFiles))
+    return;
 
  /* Change to the spool directory */
 
@@ -855,22 +883,22 @@ void DeleteLockWebpageSpoolFile(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  /* Delete the file for the backup web page. */
 
- lokfile=URLToFileName(Url);
+ lockfile=URLToFileName(Url);
 
- *lokfile='L';
+ *lockfile='L';
 
- if(unlink(lokfile))
-    PrintMessage(Inform,"Cannot unlink lock file '%s/%s/%s' [%!s].",Url->proto,Url->dir,lokfile);
+ if(unlink(lockfile))
+    PrintMessage(Inform,"Cannot unlink lock file '%s/%s/%s' [%!s].",Url->proto,Url->dir,lockfile);
 
  /* Change dir back and tidy up. */
 
- chdir("../..");
+ fchdir(fSpoolDir);
 
- free(lokfile);
+ free(lockfile);
 }
 
 
@@ -888,7 +916,7 @@ int CreateLastTimeSpoolFile(URL *Url)
  char *file;
  int exists=0;
 
- if(NoLasttimeIndex)
+ if(ConfigBoolean(NoLasttimeIndex))
     return(1);
 
  /* Change to the last time directory */
@@ -901,7 +929,7 @@ int CreateLastTimeSpoolFile(URL *Url)
  file=URLToFileName(Url);
  *file='D';
 
- if(stat(file,&buf)==-1)
+ if(stat(file,&buf))
    {
     char *name=(char*)malloc(strlen(Url->proto)+strlen(Url->dir)+strlen(file)+8);
     sprintf(name,"../%s/%s/%s",Url->proto,Url->dir,file);
@@ -924,7 +952,7 @@ int CreateLastTimeSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(file);
 
@@ -932,41 +960,6 @@ int CreateLastTimeSpoolFile(URL *Url)
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Determine if a particular URL exists in the last time spool directory.
-
-  int ExistsLastTimeSpoolFile Returns 1 if the URL exists.
-
-  URL *Url The URL to look for.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-int ExistsLastTimeSpoolFile(URL *Url)
-{
- struct stat buf;
- int i,retval=0;
- char *name=URLToFileName(Url);
-
- *name='D';
-
- for(i=0;i<=NUM_PREVTIME_DIR;i++)
-   {
-    char lasttime[40];
-
-    if(i)
-       sprintf(lasttime,"prevtime%d/%s",i,name);
-    else
-       sprintf(lasttime,"lasttime/%s",name);
-
-    if(!stat(lasttime,&buf))
-      {retval=1;break;}
-   }
-
- free(name);
-
- return(retval);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Delete a specified URL from the lasttime directory.
 
@@ -1008,7 +1001,7 @@ char *DeleteLastTimeSpoolFile(URL *Url)
           unlink(name);
          }
 
-       chdir("..");
+       fchdir(fSpoolDir);
       }
    }
 
@@ -1028,7 +1021,7 @@ void CycleLastTimeSpoolFile(void)
  struct stat buf;
  int i;
 
- /* Cycle the directories */
+ /* Cycle the lasttime/prevtime? directories */
 
  for(i=NUM_PREVTIME_DIR;i>=0;i--)
    {
@@ -1039,10 +1032,10 @@ void CycleLastTimeSpoolFile(void)
 
     /* Create it if it does not exist. */
 
-    if(lstat(lasttime,&buf))
+    if(stat(lasttime,&buf))
       {
        PrintMessage(Inform,"Directory '%s' does not exist [%!s]; creating one.",lasttime);
-       if(mkdir(lasttime,DirPerm))
+       if(mkdir(lasttime,(mode_t)ConfigInteger(DirPerm)))
           PrintMessage(Warning,"Cannot create directory '%s' [%!s].",lasttime);
       }
     else
@@ -1062,18 +1055,20 @@ void CycleLastTimeSpoolFile(void)
        dir=opendir(".");
 
        if(!dir)
-         {PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",lasttime);chdir("..");continue;}
+         {PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",lasttime);fchdir(fSpoolDir);continue;}
 
        ent=readdir(dir);  /* skip .  */
        if(!ent)
-         {PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",lasttime);closedir(dir);chdir("..");continue;}
+         {PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",lasttime);closedir(dir);fchdir(fSpoolDir);continue;}
        ent=readdir(dir);  /* skip .. */
 
        while((ent=readdir(dir)))
           if(unlink(ent->d_name))
              PrintMessage(Warning,"Cannot unlink previous time page '%s/%s' [%!s].",lasttime,ent->d_name);
 
-       chdir("..");
+       closedir(dir);
+
+       fchdir(fSpoolDir);
 
        if(rmdir(lasttime))
           PrintMessage(Warning,"Cannot unlink previous time directory '%s' [%!s].",lasttime);
@@ -1085,11 +1080,125 @@ void CycleLastTimeSpoolFile(void)
     strcpy(prevlasttime,lasttime);
    }
 
- if(mkdir("lasttime",DirPerm))
+ if(mkdir("lasttime",(mode_t)ConfigInteger(DirPerm)))
     PrintMessage(Warning,"Cannot create directory 'lasttime' [%!s].");
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Cycle the URLs from the outgoing directory to the lastout to prevout directories.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void CycleLastOutSpoolFile(void)
+{
+ char lastout[16],prevlastout[16];
+ struct stat buf;
+ int i;
+
+ /* Cycle the outgoing/lastout/prevout? directories */
+
+ for(i=NUM_PREVTIME_DIR;i>=0;i--)
+   {
+    if(i)
+       sprintf(lastout,"prevout%d",i);
+    else
+       strcpy(lastout,"lastout");
+
+    /* Create it if it does not exist. */
+
+    if(stat(lastout,&buf))
+      {
+       PrintMessage(Inform,"Directory '%s' does not exist [%!s]; creating one.",lastout);
+       if(mkdir(lastout,(mode_t)ConfigInteger(DirPerm)))
+          PrintMessage(Warning,"Cannot create directory '%s' [%!s].",lastout);
+      }
+    else
+       if(!S_ISDIR(buf.st_mode))
+          PrintMessage(Warning,"The file '%s' is not a directory.",lastout);
+
+    /* Delete the contents of the oldest one, rename the newer ones. */
+
+    if(i==NUM_PREVTIME_DIR)
+      {
+       DIR *dir;
+       struct dirent* ent;
+
+       if(chdir(lastout))
+         {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",lastout);continue;}
+
+       dir=opendir(".");
+
+       if(!dir)
+         {PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",lastout);fchdir(fSpoolDir);continue;}
+
+       ent=readdir(dir);  /* skip .  */
+       if(!ent)
+         {PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",lastout);closedir(dir);fchdir(fSpoolDir);continue;}
+       ent=readdir(dir);  /* skip .. */
+
+       while((ent=readdir(dir)))
+          if(unlink(ent->d_name))
+             PrintMessage(Warning,"Cannot unlink previous time page '%s/%s' [%!s].",lastout,ent->d_name);
+
+       closedir(dir);
+
+       fchdir(fSpoolDir);
+
+       if(rmdir(lastout))
+          PrintMessage(Warning,"Cannot unlink previous time directory '%s' [%!s].",lastout);
+      }
+    else
+       if(rename(lastout,prevlastout))
+          PrintMessage(Warning,"Cannot rename previous time directory '%s' to '%s' [%!s].",lastout,prevlastout);
+
+    strcpy(prevlastout,lastout);
+   }
+
+ if(mkdir("lastout",(mode_t)ConfigInteger(DirPerm)))
+    PrintMessage(Warning,"Cannot create directory 'lastout' [%!s].");
+
+ /* Link the files from the outgoing directory to the lastout directory. */
+
+ if(chdir("outgoing"))
+    PrintMessage(Warning,"Cannot change to directory 'outgoing' [%!s].");
+ else
+   {
+    DIR *dir;
+    struct dirent* ent;
+
+    dir=opendir(".");
+
+    if(!dir)
+       PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",lastout);
+    else
+      {
+       ent=readdir(dir);  /* skip .  */
+       if(!ent)
+          PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",lastout);
+       else
+         {
+          ent=readdir(dir);  /* skip .. */
+
+          while((ent=readdir(dir)))
+            {
+             char newname[40];
+
+             strcpy(newname,"../lastout/");
+             strcat(newname,ent->d_name);
+
+             if(link(ent->d_name,newname))
+                PrintMessage(Warning,"Cannot create lastout page '%s' [%!s].",&newname[3]);
+            }
+         }
+
+       closedir(dir);
+      }
+
+    fchdir(fSpoolDir);
+   }
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Open a file in the monitor directory to write into.
 
@@ -1114,10 +1223,10 @@ int CreateMonitorSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8],cha
 
  /* Create the monitor directory if needed and change to it */
 
- if(lstat("monitor",&buf))
+ if(stat("monitor",&buf))
    {
     PrintMessage(Inform,"Directory 'monitor' does not exist [%!s]; creating one.");
-    if(mkdir("monitor",DirPerm))
+    if(mkdir("monitor",(mode_t)ConfigInteger(DirPerm)))
       {PrintMessage(Warning,"Cannot create directory 'monitor' [%!s].");return(-1);}
    }
  else
@@ -1133,7 +1242,7 @@ int CreateMonitorSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8],cha
 
  *file='O';
 
- fd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,FilePerm);
+ fd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
  if(fd==-1)
    {PrintMessage(Warning,"Cannot create file 'monitor/%s' [%!s].",file);}
@@ -1142,7 +1251,7 @@ int CreateMonitorSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8],cha
     int ufd,mfd;
 
     *file='U';
-    ufd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,FilePerm);
+    ufd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
     if(ufd!=-1)
       {
@@ -1156,7 +1265,7 @@ int CreateMonitorSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8],cha
       }
 
     *file='M';
-    mfd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,FilePerm);
+    mfd=open(file,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
     if(mfd!=-1)
       {
@@ -1175,7 +1284,7 @@ int CreateMonitorSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8],cha
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  free(file);
 
@@ -1220,7 +1329,7 @@ long ReadMonitorTimesSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8]
  /* Check for 'O*' file. */
 
  *file='O';
- if(lstat(file,&buf))
+ if(stat(file,&buf))
     atime=0,mtime=time(NULL);
  else
     atime=buf.st_atime,mtime=buf.st_mtime;
@@ -1228,7 +1337,7 @@ long ReadMonitorTimesSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8]
  /* Check for 'M*' file for the old format. */
 
  *file='M';
- if(lstat(file,&buf) || buf.st_size<8)
+ if(stat(file,&buf) || buf.st_size<8)
    {
     int interval=7;
 
@@ -1263,7 +1372,7 @@ long ReadMonitorTimesSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8]
     else
        strcpy(HofD,"111111111111111111111111");
 
-    chdir("..");
+    fchdir(fSpoolDir);
     return(atime);
    }
 
@@ -1288,7 +1397,7 @@ long ReadMonitorTimesSpoolFile(URL *Url,char MofY[13],char DofM[32],char DofW[8]
     close(mfd);
    }
 
- chdir("..");
+ fchdir(fSpoolDir);
  return(atime);
 }
 
@@ -1335,11 +1444,11 @@ char *DeleteMonitorSpoolFile(URL *Url)
     DIR *dir=opendir(".");
 
     if(!dir)
-      {err=PrintMessage(Warning,"Cannot open current directory 'monitor' [%!s].");chdir("..");return(err);}
+      {err=PrintMessage(Warning,"Cannot open current directory 'monitor' [%!s].");fchdir(fSpoolDir);return(err);}
 
     ent=readdir(dir);  /* skip .  */
     if(!ent)
-      {err=PrintMessage(Warning,"Cannot read current directory 'monitor' [%!s].");closedir(dir);chdir("..");return(err);}
+      {err=PrintMessage(Warning,"Cannot read current directory 'monitor' [%!s].");closedir(dir);fchdir(fSpoolDir);return(err);}
     ent=readdir(dir);  /* skip .. */
 
     while((ent=readdir(dir)))
@@ -1353,7 +1462,7 @@ char *DeleteMonitorSpoolFile(URL *Url)
 
  /* Change dir back and tidy up. */
 
- chdir("..");
+ fchdir(fSpoolDir);
 
  return(err);
 }
@@ -1373,10 +1482,10 @@ int CreateTempSpoolFile(void)
 
  /* Create the outgoing directory if needed and change to it */
 
- if(lstat("temp",&buf))
+ if(stat("temp",&buf))
    {
     PrintMessage(Inform,"Directory 'temp' does not exist [%!s]; creating one.");
-    if(mkdir("temp",DirPerm))
+    if(mkdir("temp",(mode_t)ConfigInteger(DirPerm)))
       {PrintMessage(Warning,"Cannot create directory 'temp' [%!s].");return(-1);}
    }
  else
@@ -1387,7 +1496,7 @@ int CreateTempSpoolFile(void)
 
  sprintf(name,"temp/tmp.%ld",(long)getpid());
 
- fd=open(name,O_RDWR|O_CREAT|O_TRUNC|O_BINARY,FilePerm);
+ fd=open(name,O_RDWR|O_CREAT|O_TRUNC|O_BINARY,(mode_t)ConfigInteger(FilePerm));
 
  if(fd==-1)
     PrintMessage(Warning,"Cannot create temporary file '%s' [%!s].",name);
@@ -1497,3 +1606,26 @@ char *URLToFileName(URL *Url)
 
  return(file);
 }
+
+
+#if defined(__CYGWIN__)
+
+/*++++++++++++++++++++++++++++++++++++++
+  A fchdir replacement for systems that do not have one.
+
+  int fchdir Returns the error status from changing directory.
+
+  int fd The file descriptor of the spool directory (not used).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int fchdir(int fd)
+{
+ static char *spool_dir=NULL;
+
+ if(!spool_dir)
+    spool_dir=ConfigString(SpoolDir);
+
+ return(chdir(spool_dir));
+}
+
+#endif
