@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/ssl.c 1.5 1999/08/24 17:55:57 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/ssl.c 1.11 2000/08/27 09:54:47 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6.
   SSL (Secure Socket Layer) Tunneling functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1998,99 Andrew M. Bishop
+  This file Copyright 1998,99,2000 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -20,6 +20,7 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "wwwoffle.h"
 #include "errors.h"
@@ -51,7 +52,7 @@ char *SSL_Open(URL *Url)
  char *server_host=NULL;
  int server_port=0;
 
- proxy=SSLProxy;
+ proxy=ConfigStringURL(SSLProxy,Url);
  if(IsLocalNetHost(Url->host))
     proxy=NULL;
 
@@ -79,7 +80,7 @@ char *SSL_Open(URL *Url)
 
  if(server_port)
    {
-    server=OpenClientSocket(server_host,server_port,ConnectTimeout);
+    server=OpenClientSocket(server_host,server_port);
     init_buffer(server);
 
     if(server==-1)
@@ -114,17 +115,24 @@ char *SSL_Request(int client,URL *Url,Header *request_head)
    {
     char *head;
 
-    MakeRequestAuthorised(proxy,request_head);
+    MakeRequestProxyAuthorised(proxy,request_head);
+
+    request_head->size-=strlen(request_head->url)-strlen(Url->host);
+    strcpy(request_head->url,Url->host);
 
     head=HeaderString(request_head);
 
-    PrintMessage(ExtraDebug,"Outgoing Request Head (to proxy)\n%s",head);
+    PrintMessage(ExtraDebug,"Outgoing Request Head (to SSL proxy)\n%s",head);
 
     if(write_string(server,head)==-1)
-       msg=PrintMessage(Warning,"Failed to write to remote SSL proxy [%!s].");
+       msg=PrintMessage(Warning,"Failed to write to remote SSL proxy; [%!s].");
 
     free(head);
    }
+ else
+    HTMLMessageHead(client,200,"WWWOFFLE SSL OK",
+                    "Content-Type",NULL,
+                    NULL);
 
  return(msg);
 }
@@ -141,7 +149,7 @@ void SSL_Transfer(int client)
  int nfd=client>server?client+1:server+1;
  fd_set readfd;
  struct timeval tv;
- int nc,ns;
+ int n,nc,ns;
  char buffer[READ_BUFFER_SIZE];
 
  while(1)
@@ -153,10 +161,14 @@ void SSL_Transfer(int client)
     FD_SET(server,&readfd);
     FD_SET(client,&readfd);
 
-    tv.tv_sec=SocketTimeout;
+    tv.tv_sec=ConfigInteger(SocketTimeout);
     tv.tv_usec=0;
 
-    if(select(nfd,&readfd,NULL,NULL,&tv)<=0)
+    n=select(nfd,&readfd,NULL,NULL,&tv);
+
+    if(n<0 && errno==EINTR)
+       continue;
+    else if(n<=0)
        return;
 
     if(FD_ISSET(client,&readfd))
