@@ -1,5 +1,5 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle-tools.c 1.15 1999/07/08 19:28:49 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle-tools.c 1.18 2000/10/26 18:19:19 amb Exp $
 
   WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
   Tools for use in the cache for version 2.x.
@@ -33,6 +33,14 @@
 #include "errors.h"
 #include "config.h"
 
+#ifndef SPOOL_DIR
+#define SPOOL_DIR DEF_SPOOL
+#endif
+
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 /*+ Need this for Win32 to use binary mode +*/
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -54,6 +62,9 @@ static void wwwoffle_write(URL *Url);
 
 static void ls(char *file);
 
+/*+ A file descriptor for the spool directory. +*/
+int fSpoolDir=-1;
+
 
 /*++++++++++++++++++++++++++++++++++++++
   The main program
@@ -61,6 +72,7 @@ static void ls(char *file);
 
 int main(int argc,char **argv)
 {
+ char cwd[PATH_MAX+1];
  struct stat buf;
  URL *Url[2];
  int mode=0;
@@ -106,13 +118,21 @@ int main(int argc,char **argv)
    {fprintf(stderr,"Usage: wwwoffle-write <URL>\n");exit(0);}
 
  if(stat("outgoing",&buf) || !S_ISDIR(buf.st_mode))
-   {fprintf(stderr,"The wwwoffle-tools programs must be started from the spool directory\n"
-                   "There is no 'outgoing' directory here so it can't be right.\n");exit(1);}
+   {
+    chdir(SPOOL_DIR);
+    if(stat("outgoing",&buf) || !S_ISDIR(buf.st_mode))
+      {fprintf(stderr,"The wwwoffle-tools programs must be started from the spool directory\n"
+                      "There is no 'outgoing' directory here so it can't be right.\n");exit(1);}
+   }
 
  InitErrorHandler(p,0,1);
 
  umask(0);
 
+ fSpoolDir=open(".",O_RDONLY);
+ if(fSpoolDir==-1)
+    PrintMessage(Fatal,"Cannot open the spool directory '%s' [%!s]; conversion failed.",getcwd(cwd,PATH_MAX));
+
  /* Get the arguments */
 
  for(i=1;i<argc;i++)
@@ -158,6 +178,8 @@ int main(int argc,char **argv)
  else if(mode==WRITE)
     wwwoffle_write(Url[0]);
 
+ close(fSpoolDir);
+
  exit(0);
 }
 
@@ -174,7 +196,7 @@ static void wwwoffle_ls(URL *Url)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url->proto);return;}
 
  if(chdir(Url->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
  if(strcmp(Url->path,"/"))
    {
@@ -191,11 +213,11 @@ static void wwwoffle_ls(URL *Url)
     DIR *dir=opendir(".");
 
     if(!dir)
-      {PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url->proto,Url->dir);chdir("../..");return;}
+      {PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url->proto,Url->dir);fchdir(fSpoolDir);return;}
 
     ent=readdir(dir);  /* skip .  */
     if(!ent)
-      {PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url->proto,Url->dir);closedir(dir);chdir("../..");return;}
+      {PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url->proto,Url->dir);closedir(dir);fchdir(fSpoolDir);return;}
     ent=readdir(dir);  /* skip .. */
 
     while((ent=readdir(dir)))
@@ -207,7 +229,7 @@ static void wwwoffle_ls(URL *Url)
     closedir(dir);
    }
 
- chdir("../..");
+ fchdir(fSpoolDir);
 }
 
 
@@ -228,11 +250,11 @@ static void wwwoffle_ls_special(char *name)
  dir=opendir(".");
 
  if(!dir)
-   {PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",name);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot open current directory '%s' [%!s].",name);fchdir(fSpoolDir);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",name);closedir(dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot read current directory '%s' [%!s].",name);closedir(dir);fchdir(fSpoolDir);return;}
  ent=readdir(dir);  /* skip .. */
 
  while((ent=readdir(dir)))
@@ -243,7 +265,7 @@ static void wwwoffle_ls_special(char *name)
 
  closedir(dir);
 
- chdir("..");
+ fchdir(fSpoolDir);
 }
 
 
@@ -300,16 +322,16 @@ static void wwwoffle_mv(URL *Url1,URL *Url2)
    {PrintMessage(Warning,"Cannot change to directory '%s' [%!s].",Url1->proto);return;}
 
  if(chdir(Url1->dir))
-   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url1->proto,Url1->dir);chdir("..");return;}
+   {PrintMessage(Warning,"Cannot change to directory '%s/%s' [%!s].",Url1->proto,Url1->dir);fchdir(fSpoolDir);return;}
 
  dir=opendir(".");
 
  if(!dir)
-   {PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url1->proto,Url1->dir);chdir("../..");return;}
+   {PrintMessage(Warning,"Cannot open current directory '%s/%s' [%!s].",Url1->proto,Url1->dir);fchdir(fSpoolDir);return;}
 
  ent=readdir(dir);  /* skip .  */
  if(!ent)
-   {PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url1->proto,Url1->dir);closedir(dir);chdir("../..");return;}
+   {PrintMessage(Warning,"Cannot read current directory '%s/%s' [%!s].",Url1->proto,Url1->dir);closedir(dir);fchdir(fSpoolDir);return;}
  ent=readdir(dir);  /* skip .. */
 
  while((ent=readdir(dir)))
@@ -365,7 +387,7 @@ static void wwwoffle_mv(URL *Url1,URL *Url2)
 
  closedir(dir);
 
- chdir("../..");
+ fchdir(fSpoolDir);
 }
 
 
