@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 2.28 1999/08/15 14:40:14 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 2.38 2001/02/23 20:15:49 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6b.
   A user level program to interact with the server.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98,99 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000,01 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -17,7 +17,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <limits.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -30,6 +29,10 @@
 #include "sockets.h"
 #include "errors.h"
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 
 static void usage(int verbose);
 
@@ -65,9 +68,13 @@ static void add_url_list(char **links);
 static void add_url_file(char *url_file);
 
 /*+ The list of URLs or files. +*/
-char **url_file_list=NULL;
+static char **url_file_list=NULL;
+
 /*+ The number of URLs or files. +*/
-int n_url_file_list=0;
+static int n_url_file_list=0;
+
+/*+ A file descriptor for the spool directory; not used anywhere, but required for linking with spool.o. +*/
+int fSpoolDir=-1;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -77,8 +84,10 @@ int n_url_file_list=0;
 int main(int argc, char** argv)
 {
  int i;
- int recursive_depth=0,recursive_mode=0,force=0;
+ int recursive_mode=0,depth=0;
+ int force=0;
  int stylesheets=0,images=0,frames=0,scripts=0,objects=0;
+ char *config_file=NULL;
 
  Action action=None;
 
@@ -88,8 +97,6 @@ int main(int argc, char** argv)
 
  /* Parse the command line options */
 
- ConfigFile=NULL;
-
  if(argc==1)
     usage(0);
 
@@ -188,41 +195,13 @@ int main(int argc, char** argv)
        continue;
       }
 
-    if(!strcmp(argv[i],"-i"))
-      {
-       fprintf(stderr,"wwwoffle: The '-i' option is being replaced by '-gi'.\n");
-
-       if(action!=None && action!=Get)
-         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
-       action=Get;
-       if(images)
-         {fprintf(stderr,"wwwoffle: Only one '-i' argument may be given.\n"); exit(1);}
-       images=1;
-       argv[i]=NULL;
-       continue;
-      }
-
-    if(!strcmp(argv[i],"-f"))
-      {
-       fprintf(stderr,"wwwoffle: The '-f' option is being replaced by '-gf'.\n");
-
-       if(action!=None && action!=Get)
-         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
-       action=Get;
-       if(frames)
-         {fprintf(stderr,"wwwoffle: Only one '-f' argument may be given.\n"); exit(1);}
-       frames=1;
-       argv[i]=NULL;
-       continue;
-      }
-
     if(!strncmp(argv[i],"-R",2) || !strncmp(argv[i],"-r",2) || !strncmp(argv[i],"-d",2))
       {
        if(action!=None && action!=Get)
          {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
        action=Get;
 
-       if(recursive_depth)
+       if(depth)
          {fprintf(stderr,"wwwoffle: Only one '-d', '-r' or '-R' argument may be given.\n"); exit(1);}
 
        if(argv[i][1]=='d')
@@ -233,10 +212,10 @@ int main(int argc, char** argv)
           recursive_mode=3;
 
        if(argv[i][2])
-          recursive_depth=atoi(&argv[i][2]);
+          depth=atoi(&argv[i][2]);
        else
-          recursive_depth=1;
-       if(recursive_depth<=0)
+          depth=1;
+       if(depth<=0)
          {fprintf(stderr,"wwwoffle: The '-%c' argument may only be followed by a positive integer.\n",argv[i][1]); exit(1);}
 
        argv[i]=NULL;
@@ -250,7 +229,7 @@ int main(int argc, char** argv)
        if(++i>=argc)
          {fprintf(stderr,"wwwoffle: The '-p' argument requires a hostname and optionally a port number.\n"); exit(1);}
 
-       if(ConfigFile)
+       if(config_file)
          {fprintf(stderr,"wwwoffle: The '-p' and '-c' options cannot be used together.\n"); exit(1);}
 
        if((colon=strchr(argv[i],':')))
@@ -278,7 +257,7 @@ int main(int argc, char** argv)
        if(host)
          {fprintf(stderr,"wwwoffle: The '-p' and '-c' options cannot be used together.\n"); exit(1);}
 
-       ConfigFile=argv[i];
+       config_file=argv[i];
 
        argv[i-1]=NULL;
        argv[i]=NULL;
@@ -368,10 +347,10 @@ int main(int argc, char** argv)
 
  /* Initialise things. */
 
- if(!ConfigFile && !host && (env=getenv("WWWOFFLE_PROXY")))
+ if(!config_file && !host && (env=getenv("WWWOFFLE_PROXY")))
    {
     if(*env=='/')
-       ConfigFile=env;
+       config_file=env;
     else
       {
        char *colon1,*colon2;
@@ -401,10 +380,13 @@ int main(int argc, char** argv)
 
  InitErrorHandler("wwwoffle",0,1);
 
- if(ConfigFile)
+ InitConfigurationFile(config_file);
+
+ if(config_file)
    {
-    if(ReadConfigFile(2))
-       PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
+    init_buffer(2);
+    if(ReadConfigurationFile(2))
+       PrintMessage(Fatal,"Error in configuration file '%s'.",config_file);
 
     host=GetLocalHost(0);
    }
@@ -416,16 +398,16 @@ int main(int argc, char** argv)
     int socket;
     char *line=NULL;
 
-    socket=OpenClientSocket(host?host:"localhost",port?port:WWWOFFLE_Port,ConnectTimeout);
+    socket=OpenClientSocket(host?host:"localhost",port?port:ConfigInteger(WWWOFFLE_Port));
     init_buffer(socket);
 
     if(socket==-1)
-       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:WWWOFFLE_Port);
+       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:ConfigInteger(WWWOFFLE_Port));
 
     /* Send the message. */
 
-    if(PassWord)
-       write_formatted(socket,"WWWOFFLE PASSWORD %s\r\n",PassWord);
+    if(ConfigString(PassWord))
+       write_formatted(socket,"WWWOFFLE PASSWORD %s\r\n",ConfigString(PassWord));
 
     if(action==Online)
        write_string(socket,"WWWOFFLE ONLINE\r\n");
@@ -458,12 +440,8 @@ int main(int argc, char** argv)
  else if(action==Get)
    {
     URL *Url;
-    struct stat buf;
     char *refresh=NULL;
-
-    if(recursive_depth || recursive_mode || force || stylesheets || images || frames || scripts || objects)
-       refresh=CreateRefreshPath(recursive_depth,recursive_mode,force,
-                                 stylesheets,images,frames,scripts,objects);
+    struct stat buf;
 
     for(i=0;i<n_url_file_list;i++)
        if(strcmp(url_file_list[i],"-") && stat(url_file_list[i],&buf))
@@ -471,16 +449,46 @@ int main(int argc, char** argv)
           int socket;
           char buffer[READ_BUFFER_SIZE];
 
-          socket=OpenClientSocket(host?host:"localhost",port?port:HTTP_Port,ConnectTimeout);
+          socket=OpenClientSocket(host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
           init_buffer(socket);
 
           if(socket==-1)
-             PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:HTTP_Port);
+             PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
 
           Url=SplitURL(url_file_list[i]);
 
-          if(refresh)
+          if(recursive_mode || depth || force || stylesheets || images || frames || scripts || objects)
+            {
+             char *limit="";
+
+             if(recursive_mode==3)
+                limit="";
+             else if(recursive_mode==2)
+               {
+                limit=(char*)malloc(strlen(Url->proto)+strlen(Url->host)+5);
+                sprintf(limit,"%s://%s/",Url->proto,Url->host);
+               }
+             else if(recursive_mode==1)
+               {
+                char *p;
+                limit=(char*)malloc(strlen(Url->proto)+strlen(Url->host)+strlen(Url->path)+5);
+                sprintf(limit,"%s://%s%s",Url->proto,Url->host,Url->path);
+                p=limit+strlen(limit)-1;
+                while(p>limit && *p!='/')
+                   *p--=0;
+               }
+             else
+                depth=0;
+
+             refresh=CreateRefreshPath(Url,limit,depth,
+                                       force,
+                                       stylesheets,images,frames,scripts,objects);
+
+             if(*limit)
+                free(limit);
+
              printf("Getting: %s (with recursive options).\n",Url->name);
+            }
           else
              printf("Getting: %s\n",Url->name);
 
@@ -492,14 +500,16 @@ int main(int argc, char** argv)
              userpass2=Base64Encode(userpass1,strlen(userpass1));
 
              if(refresh)
-                write_formatted(socket,"GET %s/?%s HTTP/1.0\r\n"
+                write_formatted(socket,"GET %s HTTP/1.0\r\n"
                                        "Authorization: Basic %s\r\n"
+                                       "Pragma: wwwoffle\r\n"
                                        "Accept: */*\r\n"
                                        "\r\n",
-                                refresh,Url->name,userpass2);
+                                refresh,userpass2);
              else
                 write_formatted(socket,"GET %s HTTP/1.0\r\n"
                                        "Authorization: Basic %s\r\n"
+                                       "Pragma: wwwoffle\r\n"
                                        "Accept: */*\r\n"
                                        "\r\n",
                                 Url->name,userpass2);
@@ -510,12 +520,14 @@ int main(int argc, char** argv)
           else
             {
              if(refresh)
-                write_formatted(socket,"GET %s/?%s HTTP/1.0\r\n"
+                write_formatted(socket,"GET %s HTTP/1.0\r\n"
+                                       "Pragma: wwwoffle\r\n"
                                        "Accept: */*\r\n"
                                        "\r\n",
-                                refresh,Url->name);
+                                refresh);
              else
                 write_formatted(socket,"GET %s HTTP/1.0\r\n"
+                                       "Pragma: wwwoffle\r\n"
                                        "Accept: */*\r\n"
                                        "\r\n",
                                 Url->name);
@@ -526,6 +538,7 @@ int main(int argc, char** argv)
 
           CloseSocket(socket);
 
+          free(refresh);
           FreeURL(Url);
          }
        else if(!strcmp(url_file_list[i],"-") || S_ISREG(buf.st_mode))
@@ -605,11 +618,11 @@ int main(int argc, char** argv)
     char buffer[READ_BUFFER_SIZE];
     char *data=(char*)malloc(READ_BUFFER_SIZE+1);
 
-    socket=OpenClientSocket(host?host:"localhost",port?port:HTTP_Port,ConnectTimeout);
+    socket=OpenClientSocket(host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
     init_buffer(socket);
 
     if(socket==-1)
-       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:HTTP_Port);
+       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
 
     Url=SplitURL(url_file_list[0]);
 
@@ -624,6 +637,13 @@ int main(int argc, char** argv)
        data=(char*)realloc((void*)data,length+READ_BUFFER_SIZE+1);
       }
 
+    if(action==Post)
+      {
+       data[length]=0;
+       data=URLRecodeFormArgs(data);
+       length=strlen(data);
+      }
+
     if(Url->user)
       {
        char *userpass1=(char*)(malloc(strlen(Url->user)+(Url->pass?strlen(Url->pass):1)+3)),*userpass2;
@@ -636,6 +656,7 @@ int main(int argc, char** argv)
                                  "Authorization: Basic %s\r\n"
                                  "Content-Type: application/x-www-form-urlencoded\r\n"
                                  "Content-Length: %d\r\n"
+                                 "Pragma: wwwoffle\r\n"
                                  "Accept: */*\r\n"
                                  "\r\n",
                           Url->name,userpass2,length);
@@ -643,6 +664,7 @@ int main(int argc, char** argv)
           write_formatted(socket,"PUT %s HTTP/1.0\r\n"
                                  "Authorization: Basic %s\r\n"
                                  "Content-Length: %d\r\n"
+                                 "Pragma: wwwoffle\r\n"
                                  "Accept: */*\r\n"
                                  "\r\n",
                           Url->name,userpass2,length);
@@ -656,12 +678,14 @@ int main(int argc, char** argv)
           write_formatted(socket,"POST %s HTTP/1.0\r\n"
                                  "Content-Type: application/x-www-form-urlencoded\r\n"
                                  "Content-Length: %d\r\n"
+                                 "Pragma: wwwoffle\r\n"
                                  "Accept: */*\r\n"
                                  "\r\n",
                           Url->name,length);
        else
           write_formatted(socket,"PUT %s HTTP/1.0\r\n"
                                  "Content-Length: %d\r\n"
+                                 "Pragma: wwwoffle\r\n"
                                  "Accept: */*\r\n"
                                  "\r\n",
                           Url->name,length);
@@ -689,11 +713,11 @@ int main(int argc, char** argv)
     if(!n_url_file_list)
        PrintMessage(Fatal,"No URL specified to output.");
 
-    socket=OpenClientSocket(host?host:"localhost",port?port:HTTP_Port,ConnectTimeout);
+    socket=OpenClientSocket(host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
     init_buffer(socket);
 
     if(socket==-1)
-       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:HTTP_Port);
+       PrintMessage(Fatal,"Cannot open connection to wwwoffle server %s port %d.",host?host:"localhost",port?port:ConfigInteger(HTTP_Port));
 
     Url=SplitURL(url_file_list[0]);
 
@@ -708,9 +732,9 @@ int main(int argc, char** argv)
        userpass2=Base64Encode(userpass1,strlen(userpass1));
 
        write_formatted(socket,"GET %s HTTP/1.0\r\n"
+                              "Authorization: Basic %s\r\n"
                               "Pragma: wwwoffle\r\n"
                               "Accept: */*\r\n"
-                              "Authorization: Basic %s\r\n"
                               "\r\n",
                        Url->name,userpass2);
 
@@ -783,8 +807,8 @@ static void usage(int verbose)
 
  if(verbose)
     fprintf(stderr,
-            "(c) Andrew M. Bishop 1996,97,98,99 [       amb@gedanken.demon.co.uk ]\n"
-            "                                   [http://www.gedanken.demon.co.uk/]\n"
+            "(c) Andrew M. Bishop 1996,97,98,99,2000 [       amb@gedanken.demon.co.uk ]\n"
+            "                                        [http://www.gedanken.demon.co.uk/]\n"
             "\n");
 
  fprintf(stderr,
