@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 2.18 1999/08/08 09:22:33 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 2.27 2001/02/23 20:15:49 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.5.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.6b.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98,99 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000,01 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <signal.h>
+#include <grp.h>
 
 #include "version.h"
 #include "wwwoffle.h"
@@ -55,10 +56,14 @@ int http_fd=-1,                 /*+ for the HTTP connections. +*/
 /*+ The online / offline /autodial status. +*/
 int online=0;
 
-/*+ The current number active servers +*/
+/*+ The current number of active servers +*/
 int n_servers=0,                /*+ in total. +*/
     n_fetch_servers=0;          /*+ fetching a page. +*/
 
+/*+ The maximum number of servers +*/
+int max_servers=0,              /*+ in total. +*/
+    max_fetch_servers=0;        /*+ fetching a page. +*/
+
 /*+ The wwwoffle client file descriptor when fetching. +*/
 int fetch_fd=-1;
 
@@ -71,6 +76,9 @@ int fetch_pids[MAX_FETCH_SERVERS];
 /*+ The current status, fetching or not. +*/
 int fetching=0;
 
+/*+ The file descriptor of the sppol directory. +*/
+int fSpoolDir=-1;
+
 /*+ Set to 1 when the demon is to shutdown. +*/
 static int closedown=0;
 
@@ -96,6 +104,7 @@ int main(int argc, char** argv)
  int i;
  int err;
  struct stat buf;
+ char *config_file=NULL;
 
  /* Parse the command line options */
 
@@ -127,7 +136,7 @@ int main(int argc, char** argv)
        if(++i>=argc)
          {fprintf(stderr,"wwwoffled: The '-c' option requires a filename.\n"); exit(1);}
 
-       ConfigFile=argv[i];
+       config_file=argv[i];
        continue;
       }
 
@@ -142,67 +151,91 @@ int main(int argc, char** argv)
  for(i=0;i<MAX_SERVERS;i++)
     server_pids[i]=0;
 
- InitErrorHandler("wwwoffled",1,!detached);
+ InitErrorHandler("wwwoffled",0,1); /* use stderr and not syslog to start with. */
 
- if(ReadConfigFile(2))
-    PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
+ InitConfigurationFile(config_file);
+
+ init_buffer(2);
+ if(ReadConfigurationFile(2))
+    PrintMessage(Fatal,"Error in configuration file '%s'.",config_file);
+
+ InitErrorHandler("wwwoffled",ConfigInteger(UseSyslog),1); /* enable syslog if requested. */
 
  PrintMessage(Important,"WWWOFFLE Demon Version %s started.",WWWOFFLE_VERSION);
- PrintMessage(Inform,"WWWOFFLE Read Configuration File.");
+ PrintMessage(Inform,"WWWOFFLE Read Configuration File '%s'.",config_file);
 
- if(WWWOFFLE_Gid != -1)
+ if(ConfigInteger(WWWOFFLE_Gid) != -1)
    {
-    if(setgid(WWWOFFLE_Gid)<0)
-       PrintMessage(Fatal,"Cannot set group id to %d [%!s].",WWWOFFLE_Gid);
-    if(setegid(WWWOFFLE_Gid)<0)
-       PrintMessage(Fatal,"Cannot set effective group id to %d [%!s].",WWWOFFLE_Gid);
+#if !defined(__CYGWIN__)
+    if(getuid()==0)
+       if(setgroups(0, NULL)<0)
+          PrintMessage(Fatal,"Cannot clear supplementary group list [%!s].");
+#endif
+
+    if(setgid(ConfigInteger(WWWOFFLE_Gid))<0)
+       PrintMessage(Fatal,"Cannot set group id to %d [%!s].",ConfigInteger(WWWOFFLE_Gid));
+    if(setegid(ConfigInteger(WWWOFFLE_Gid))<0)
+       PrintMessage(Fatal,"Cannot set effective group id to %d [%!s].",ConfigInteger(WWWOFFLE_Gid));
    }
 
- if(WWWOFFLE_Uid != -1)
+ if(ConfigInteger(WWWOFFLE_Uid) != -1)
    {
-    if(setuid(WWWOFFLE_Uid)<0)
-       PrintMessage(Fatal,"Cannot set user id to %d [%!s].",WWWOFFLE_Uid);
-    if(seteuid(WWWOFFLE_Uid)<0)
-       PrintMessage(Fatal,"Cannot set effective user id to %d [%!s].",WWWOFFLE_Uid);
+    if(setuid(ConfigInteger(WWWOFFLE_Uid))<0)
+       PrintMessage(Fatal,"Cannot set user id to %d [%!s].",ConfigInteger(WWWOFFLE_Uid));
+    if(seteuid(ConfigInteger(WWWOFFLE_Uid))<0)
+       PrintMessage(Fatal,"Cannot set effective user id to %d [%!s].",ConfigInteger(WWWOFFLE_Uid));
    }
 
- if(WWWOFFLE_Gid != -1 || WWWOFFLE_Uid != -1)
+ if(ConfigInteger(WWWOFFLE_Gid) != -1 || ConfigInteger(WWWOFFLE_Uid) != -1)
     PrintMessage(Inform,"Running with uid=%d, gid=%d.",getuid(),getgid());
 
- http_fd=OpenServerSocket(HTTP_Port);
+ http_fd=OpenServerSocket(ConfigInteger(HTTP_Port));
  if(http_fd==-1)
     PrintMessage(Fatal,"Cannot create HTTP server socket.");
 
- wwwoffle_fd=OpenServerSocket(WWWOFFLE_Port);
+ wwwoffle_fd=OpenServerSocket(ConfigInteger(WWWOFFLE_Port));
  if(wwwoffle_fd==-1)
     PrintMessage(Fatal,"Cannot create WWWOFFLE server socket.");
 
  umask(0);
 
- if(stat(SpoolDir,&buf))
+ /* Create, Change to and open the spool directory. */
+
+ if(stat(ConfigString(SpoolDir),&buf))
    {
-    err=mkdir(SpoolDir,DirPerm);
+    err=mkdir(ConfigString(SpoolDir),(mode_t)ConfigInteger(DirPerm));
     if(err==-1 && errno!=EEXIST)
-       PrintMessage(Fatal,"Cannot create spool directory %s [%!s].",SpoolDir);
-    stat(SpoolDir,&buf);
+       PrintMessage(Fatal,"Cannot create spool directory %s [%!s].",ConfigString(SpoolDir));
+    stat(ConfigString(SpoolDir),&buf);
    }
 
  if(!S_ISDIR(buf.st_mode))
     PrintMessage(Fatal,"The spool directory %s is not a directory.",SpoolDir);
 
- err=chdir(SpoolDir);
+ err=chdir(ConfigString(SpoolDir));
  if(err==-1)
-    PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].",SpoolDir);
+    PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].",ConfigString(SpoolDir));
+
+#if !defined(__CYGWIN__)
+ fSpoolDir=open(".",O_RDONLY);
+ if(fSpoolDir==-1)
+    PrintMessage(Fatal,"Cannot open the spool directory '%s' [%!s].",ConfigString(SpoolDir));
+#endif
+
+ /* Detach from terminal */
 
  if(detached)
    {
     demoninit();
     PrintMessage(Important,"Detached from terminal and changed pid to %d.",getpid());
-    InitErrorHandler("wwwoffled",1,!detached); /* pid changes after detaching. */
+    InitErrorHandler("wwwoffled",1,0); /* pid changes after detaching, enable syslog, disable stderr. */
    }
 
  install_sighandlers();
 
+ max_servers=ConfigInteger(MaxServers);
+ max_fetch_servers=ConfigInteger(MaxFetchServers);
+
  /* Loop around waiting for connections. */
 
  do
@@ -221,7 +254,7 @@ int main(int argc, char** argv)
 
     FD_SET(wwwoffle_fd,&readfd);
 
-    if(n_servers<MaxServers)
+    if(n_servers<max_servers)
        FD_SET(http_fd,&readfd);
 
     tv.tv_sec=10;
@@ -289,7 +322,7 @@ int main(int argc, char** argv)
 
        PrintMessage(Important,"WWWOFFLE Re-reading Configuration File.");
 
-       if(ReadConfigFile(-1))
+       if(ReadConfigurationFile(-1))
           PrintMessage(Warning,"Error in configuration file; keeping old values.");
 
        PrintMessage(Important,"WWWOFFLE Finished Re-reading Configuration File.");
@@ -313,7 +346,7 @@ int main(int argc, char** argv)
          else if(WIFSIGNALED(status))
             exitval=-WTERMSIG(status);
 
-         for(i=0;i<MaxServers;i++)
+         for(i=0;i<max_servers;i++)
             if(server_pids[i]==pid)
               {
                n_servers--;
@@ -329,7 +362,7 @@ int main(int argc, char** argv)
               }
 
          /* Check if the child that terminated is one of the fetching wwwoffles */
-         for(i=0;i<MaxFetchServers;i++)
+         for(i=0;i<max_fetch_servers;i++)
             if(fetch_pids[i]==pid)
               {
                n_fetch_servers--;
@@ -354,7 +387,7 @@ int main(int argc, char** argv)
     /* The select timed out or we got a signal. If we are currently fetching,
        start fetch servers to look for jobs in the spool directory. */
 
-    while(fetching && n_fetch_servers<MaxFetchServers && n_servers<MaxServers)
+    while(fetching && n_fetch_servers<max_fetch_servers && n_servers<max_servers)
        ForkServer(fetch_fd,0);
 
     if(fetch_fd!=-1 && !fetching && n_fetch_servers==0)
@@ -364,6 +397,8 @@ int main(int argc, char** argv)
        fetch_fd=-1;
 
        PrintMessage(Important,"WWWOFFLE Fetch finished.");
+
+       ForkRunModeScript(ConfigString(RunFetch),"fetch","stop",fetch_fd);
       }
    }
  while(!closedown);
@@ -390,7 +425,7 @@ int main(int argc, char** argv)
        else if(WIFSIGNALED(status))
           exitval=-WTERMSIG(status);
 
-       for(i=0;i<MaxServers;i++)
+       for(i=0;i<max_servers;i++)
           if(server_pids[i]==pid)
             {
              n_servers--;
@@ -408,6 +443,8 @@ int main(int argc, char** argv)
 
  PrintMessage(Important,"Exiting.");
 
+ close(fSpoolDir);
+
  return(0);
 }
 
@@ -446,7 +483,7 @@ static void usage(int verbose)
             "                         0 <= log-level <= %d (default in config file).\n"
             "\n"
             "   -p                  : Print the pid of wwwoffled on stdout (unless -d).\n"
-            "\n",ConfigFile,Fatal+1);
+            "\n",ConfigurationFileName(),Fatal+1);
 
  exit(0);
 }
